package com.cloudbility.common.util;

import java.io.*;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;

public class CSVUtil {

    public static List<String[]> readCsvFile(InputStream inputStream) throws IOException {
        return readCsvFile(inputStream, "UTF-8");
    }

    /**
     * 返回cvs表，List第一个元素是表头
     *
     * @param inputStream
     * @return
     * @throws IOException
     */
    public static List<String[]> readCsvFile(InputStream inputStream, String charsetName) throws IOException {
        InputStreamReader isr = new InputStreamReader(inputStream, charsetName);
        BufferedReader reader = new BufferedReader(isr);
        List<String[]> list = new ArrayList<String[]>();
        try {
            String line = reader.readLine();
            String[] headers = line.split(",");
            int cols = headers.length;
            list.add(headers);
            while ((line = reader.readLine()) != null) {
                String[] parts = line.split(",", cols);
                list.add(parts);
            }
            return list;
        } catch (IOException e) {
            throw e;
        }
    }

    /**
     * @param inputStream
     * @param charsetName
     * @return
     * @throws IOException
     */
    public static List<String[]> readCsvFileWithCommaField(InputStream inputStream, String charsetName) throws IOException {
        InputStreamReader isr = new InputStreamReader(inputStream, charsetName);
        BufferedReader reader = new BufferedReader(isr);
        List<String[]> list = new ArrayList<String[]>();
        try {
            String line = null;
            while ((line = reader.readLine()) != null) {
                List<String> fileds = parseLineWithinCommaField(line);
                list.add(fileds.toArray(new String[0]));
            }
            return list;
        } catch (IOException e) {
            throw e;
        }
    }

    public static List<String> parseLineWithinCommaField(String line) {
        List<String> fields = new ArrayList<>();
        int partStart = 0;
        boolean quoted = false;
        for (int i = 0; i < line.length(); i++) {
            char c = line.charAt(i);
            if ('\"' == c) {
                if (quoted) {
                    fields.add(line.substring(partStart, i));
                    quoted = false;
                    partStart = -1;
                } else {
                    quoted = true;
                    partStart = i + 1;
                }
                continue;
            }
            if (',' == c) {
                if (!quoted) {
                    if (partStart >= 0) {
                        fields.add(line.substring(partStart, i));
                    }
                    partStart = i + 1;
                }
            }
        }
        if (partStart > 0) {
            fields.add(line.substring(partStart, line.length()));
        }
        return fields;
    }


    /**
     * 导出生成csv格式的文件
     *
     * @param out        输出流
     * @param titles     csv格式头文
     * @param properties 需要导出的数据实体的属性，注意与title一一对应
     * @param list       需要导出的对象集合
     * @throws IOException
     * @throws IllegalAccessException
     * @throws IllegalArgumentException
     * @author ccg
     */
    public static <T> void exportCsv(OutputStream out, String charset, String[] titles, String[] properties, List<T> list) throws IOException, IllegalArgumentException, IllegalAccessException {

        //补充bom头，防止excel不识别utf8乱码
        byte[] uft8bom={(byte)0xef,(byte)0xbb,(byte)0xbf};
        out.write(uft8bom);
        //构建输出流，同时指定编码
        OutputStreamWriter ow = new OutputStreamWriter(out, charset);
        //csv文件是逗号分隔，除第一个外，每次写入一个单元格数据后需要输入逗号
        for (String title : titles) {
            ow.write(title);
            ow.write(",");
        }
        //写完文件头后换行
        ow.write("\r\n");
        //写内容
        for (Object obj : list) {
            //利用反射获取所有字段
            for (String property : properties) {
                try {
                    Field field = obj.getClass().getDeclaredField(property);
                    field.setAccessible(true);
                    Object v = field.get(obj);
                    ow.write(v == null ? " " : v.toString());
                    ow.write(",");
                } catch (NoSuchFieldException e) {
                    ow.write(",");
                }
            }
            //写完一行换行
            ow.write("\r\n");
        }
        ow.flush();
        ow.close();
    }

}
