package com.cloudbility.openapi;

import com.cloudbility.openapi.enums.zstack.*;
import com.cloudbility.openapi.request.zstack.VMCreateSystemTags;
import com.cloudbility.openapi.response.*;
import com.cloudbility.openapi.vo.*;
import com.cloudbility.openapi.vo.monitor.MetricSingletonMap;
import com.cloudbility.openapi.vo.monitor.MonitorMetric;
import org.zstack.sdk.InstanceOfferingInventory;
import org.zstack.sdk.VmInstanceInventory;

import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * 镜像类主机操作接口，主要给安恒使用，未来会改为微服务的形式提供
 */
public interface ImageHostApi {

    /**
     * 获取主机状态
     * @param teamId
     * @param hostIds
     * @return
     * @throws OpenapiException
     */
    Map<Long, String> getHostStatusMap(Long teamId, List<Long> hostIds) throws OpenapiException;

    /**
     * 获取主机信息
     * @param hostId
     * @return
     * @throws OpenapiException
     */
    CloudHostVo getHost(Long hostId) throws OpenapiException;

    /**
     * 创建虚拟机
     * @param teamId                  团队ID
     * @param cloudId                 云账户ID
     * @param scope                   云账户来源，Portal|Console
     * @param projectId               项目ID，非必须，华为云需要该参数
     * @param regionId                区域ID
     * @param zoneId                  可用区ID
     * @param imageId                 镜像ID
     * @param instanceType            实例规格 如ecs.g6.large
     * @param securityGroupId         安全组ID
     * @param vpcId                   VPC ID
     * @param subnetId                子网ID
     * @param instanceName            实例名称
     * @param description             实例描述，非必须
     * @param password                实例的初始化密码，非必须
     * @param instanceCount           购买数量，默认为1
     * @param instanceChargeType      实例收费类型，PrePaid(预付费，即包年包月)|PostPaid(后付费，即按小时计费)
     * @param internetChargeType      网络类型，PayByBandwidth(按固定带宽计费)|PayByTraffic(按带宽流量计费)
     * @param internetMaxBandwidthOut 最大公网出口带宽，单位：Mbps，默认值：0Mbps，当公网带宽为0，则不允许分配公网IP
     * @param period                  购买时长，默认1个月
     * @param periodUnit              时长单位，Week|Month（默认）
     * @param systemDisk              系统盘，非必须
     * @param dataDisks               数据盘，非必须
     * @param clientToken             用于保证请求幂等性的字符串，非必须
     * @return
     */
    CloudHostResult createHost(Long teamId, Long cloudId, Scope scope, String projectId, String regionId, String zoneId, String imageId, String instanceType, String securityGroupId,
                               String vpcId, String subnetId, String instanceName, String description, String password, Integer instanceCount,
                               CloudInstanceChargeType instanceChargeType, CloudInternetChargeType internetChargeType, Integer internetMaxBandwidthOut, Integer period, String periodUnit,
                               SystemDiskVo systemDisk, List<DataDiskVo> dataDisks, String clientToken) throws OpenapiException;

    /**
     * 查询购买虚拟机每个月的预估费用
     *
     * @param vendor                  供应商
     * @param cloudId                 云账户ID
     * @param scope                   云账户来源，Portal|Console
     * @param projectId               项目ID，非必须，华为云需要该参数
     * @param regionId                区域ID
     * @param zoneId                  可用区ID，非必须
     * @param imageId                 镜像ID，非必须
     * @param instanceType            实例规格 如ecs.g6.large
     * @param securityGroupId         安全组ID，非必须
     * @param vpcId                   VPC ID，非必须
     * @param subnetId                子网ID，非必须
     * @param instanceName            实例名称
     * @param description             实例描述，非必须
     * @param password                实例的初始化密码，非必须
     * @param instanceCount           购买数量，默认为1
     * @param instanceChargeType      实例收费类型，PrePaid(预付费，即包年包月)|PostPaid(后付费，即按小时计费)
     * @param internetChargeType      网络类型，PayByBandwidth(按固定带宽计费)|PayByTraffic(按带宽流量计费)
     * @param internetMaxBandwidthOut 最大公网出口带宽，单位：Mbps，默认值：0Mbps，当公网带宽为0，则不允许分配公网IP
     * @param period                  购买时长，默认1个月
     * @param periodUnit              时长单位，Week|Month（默认）
     * @param systemDisk              系统盘，非必须
     * @param dataDisks               数据盘，非必须
     * @return
     */
    Double inquiryPriceMonthly(String vendor, Long cloudId, Scope scope, String projectId, String regionId, String zoneId, String imageId, String instanceType, String securityGroupId,
                               String vpcId, String subnetId, String instanceName, String description, String password, Integer instanceCount,
                               CloudInstanceChargeType instanceChargeType, CloudInternetChargeType internetChargeType, Integer internetMaxBandwidthOut, Integer period, String periodUnit,
                               SystemDiskVo systemDisk, List<DataDiskVo> dataDisks) throws OpenapiException;

    /**
     * 销毁虚拟机
     *
     * @param hostId 主机ID
     */
    void destroyHost(long hostId) throws OpenapiException;

    /**
     * 启动虚拟机
     *
     * @param hostId
     * @throws OpenapiException
     */
    void startHost(long hostId) throws OpenapiException;

    /**
     * 关闭虚拟机
     *
     * @param hostId
     * @throws OpenapiException
     */
    void stopHost(long hostId) throws OpenapiException;

    /**
     * 重启虚拟机
     *
     * @param hostId
     * @param force  是否强制重启
     * @throws OpenapiException
     */
    void restartHost(long hostId, Boolean force) throws OpenapiException;

    /**
     * 续费虚拟机
     *
     * @param hostId
     * @param period      包年包月续费时长
     * @param periodUnit  续费时长的时间单位  Week|Month（默认）|Year
     * @param clientToken 用于保证请求幂等性的字符串
     * @throws OpenapiException
     */
    void renewHost(long hostId, int period, RenewPeriodUnit periodUnit, String clientToken) throws OpenapiException;

    /**
     * 变更主机规格(CPU和内存)
     *
     * @param hostId       主机ID
     * @param instanceType 实例规格 如ecs.g6.large
     * @param clientToken  用于保证请求幂等性的字符串
     * @return
     * @throws OpenapiException
     */
    CloudHostVo modifyHostSpec(long hostId, String instanceType, String clientToken) throws OpenapiException;

    /**
     * 扩容一块云盘
     *
     * @param hostId      主机ID
     * @param diskId      磁盘ID,如d-bp67acfmxazb4p
     * @param newSize     单位为GB,指定的新云盘容量必须比原云盘容量大
     * @param clientToken 用于保证请求幂等性的字符串
     */
    DataDiskVo resizeDisk(long hostId, String diskId, int newSize, String clientToken) throws OpenapiException;

    /**
     * 扩容实例规格和磁盘
     *
     * @param hostId       必选，主机ID
     * @param instanceType 非必选，实例规格
     * @param diskId       非必选，磁盘ID
     * @param newSize      非必选，磁盘大小
     * @param clientToken  用于保证请求幂等性的字符串
     * @return
     * @throws OpenapiException
     */
    CloudHostVo expandHostAndDisk(long hostId, String instanceType, String diskId, Integer newSize, String clientToken) throws OpenapiException;

    CloudHostResult createZStackHost(String clientToken, Long teamId, Long cloudId, Scope scope, String zoneId, String clusterId, String hostId,
                                     String imageId, String instanceOfferingId, List<String> networkIds, String defaultNetworkId,
                                     String instanceName, String description, String hostName, String rootDiskOfferingId,
                                     Long rootDiskSize, String primaryStorageIdForRootVolume, VolumeProvisioningStrategyType rootVolumeProvisioningStrategy,
                                     List<String> dataDiskOfferingIds, VolumeProvisioningStrategyType dataVolumeProvisioningStrategy,
                                     VMCreateSystemTags systemTags, VMCreateStrategyType createStrategy, Integer instanceCount) throws OpenapiException;

    ZStackHostSpecVo createZStackHostSpec(String clientToken, Long cloudId, Scope scope, String name, int cpuNum, long memorySizeInMb,
                                          HostAllocatorStrategyType allocatorStrategy, String description, Integer maxInstancePerHost,
                                          Long volumeTotalBandwidthInMb, Long volumeReadBandwidthInMb, Long volumeWriteBandwidthInMb,
                                          Long networkInboundBandwidthInKb, Long networkOutboundBandwidthInKb,
                                          HostAllocatorStrategyMode minimumCPUUsageHostAllocatorStrategy,
                                          HostAllocatorStrategyMode minimumMemoryUsageHostAllocatorStrategy) throws OpenapiException;

    ZStackHostSpecListResult listZStackHostSpecs(Long cloudId, Scope scope, Set<String> instanceOfferingIds, String name,
                                                 Integer cpuNum, Long memorySizeInMb, HostAllocatorStrategyType allocatorStrategy,
                                                 UsingStateType state) throws OpenapiException;

    ZStackPhysicalHostListResult listZStackPhysicalHosts(Long cloudId, Scope scope) throws OpenapiException;

    ZStackPhysicalResourceSummary getZStackPhysicalResourceSummary(Long cloudId, Scope scope) throws OpenapiException;

    void modifyZStackDefaultNetwork(String clientToken, Long hostId, String defaultNetworkId, Boolean reboot) throws OpenapiException;

    InstanceOfferingInventory changeZStackInstanceOfferingState(String clientToken, Long cloudId, Scope scope, String instanceOfferingId, UsingStateEventType stateEvent) throws OpenapiException;

    VmInstanceInventory attachZStackL3NetworkToHost(String clientToken, Long hostId, String l3NetworkId, String staticIp) throws OpenapiException;

    void deleteZStackInstanceOffering(String clientToken, Long cloudId, Scope scope, String instanceOfferingId, DeleteModeType deleteMode) throws OpenapiException;

    ZStackVmNicInfoResult getZStackVmNicInfo(Long cloudId, Scope scope, String instanceId) throws OpenapiException;

    MetricSingletonMap getHostLastMetricData(long hostId, MonitorMetric... monitorMetric) throws OpenapiException;

}
