package com.cloudbility.openapi;

import com.cloudbility.openapi.response.OpsPolicyGroupResult;
import com.cloudbility.openapi.response.OpsPolicyItemResult;
import com.cloudbility.openapi.response.OpsPolicyResult;
import com.cloudbility.openapi.vo.*;

import java.util.Map;

public interface OpsPolicyApi {

    /**
     * 根据策略组id获取子策略列表
     *
     * @param groupId 策略组id
     * @return
     * @throws OpenapiException
     */
    OpsPolicyResult getOpsPoliciesByGroupId(Long groupId) throws OpenapiException;

    /**
     * 根据云账户id获取子策略列表
     *
     * @param cloudId 云账户id
     * @return
     * @throws OpenapiException
     */
    OpsPolicyResult getOpsPoliciesByCloudId(Long cloudId) throws OpenapiException;

    /**
     * 获取默认运维策略条目
     *
     * @return
     */
    OpsPolicyItemResult getDefaultOpsPolicyItems() throws OpenapiException;


    /**
     * 获取子策略详情
     *
     * @param policyId 策略id
     * @return
     * @throws OpenapiException
     */
    OpsPolicyDetailVo getOpsPolicyDetail(Long policyId) throws OpenapiException;

    /**
     * 获取子策略的条目
     *
     * @param policyId 子策略ID
     * @return
     * @throws OpenapiException
     */
    OpsPolicyItemResult getOpsPolicyItems(Long policyId) throws OpenapiException;

    /**
     * 获取子策略的条目信息
     *
     * @param policyId
     * @param itemIdentity
     * @return
     * @throws OpenapiException
     */
    OpsPolicyItemVo getOpsPolicyItem(Long policyId, String itemIdentity) throws OpenapiException;


    /**
     * 获取策略组详情
     *
     * @param policyGroupId
     * @return
     * @throws OpenapiException
     */
    OpsPolicyGroupDetailVo getOpsPolicyGroupDetail(Long policyGroupId) throws OpenapiException;

    /**
     * 获取团队策略组列表
     *
     * @param teamId
     * @return
     * @throws OpenapiException
     */
    OpsPolicyGroupResult getOpsPolicyGroupsByTeamId(Long teamId) throws OpenapiException;

    /*--------------------------------------------------------------------------------------------------------------------*/


    /**
     * 创建策略组
     *
     * @param teamId          团队ID
     * @param policyGroupName 策略组名称
     * @param hostIds         关联主机ID列表,用逗号连接
     * @return
     * @throws OpenapiException
     */
    OpsPolicyGroupVo createOpsPolicyGroup(Long teamId, String policyGroupName, String hostIds) throws OpenapiException;

    /**
     * 创建子策略
     *
     * @param policyGroupId 策略组ID
     * @param policyName    子策略名称
     * @param policyDesc    子策略描述
     * @param priority      子策略优先级
     * @param roleIds
     * @param departmentIds
     * @param memberIds
     * @return
     * @throws OpenapiException
     */
    OpsPolicyVo createOpsPolicy(Long policyGroupId, String policyName, String policyDesc, Integer priority, String roleIds, String departmentIds, String memberIds) throws OpenapiException;

    /**
     * 添加主机到策略组
     *
     * @param policyGroupId 策略组ID
     * @param hostIds       主机ID列表，用逗号连接
     * @return
     * @throws OpenapiException
     */
    Boolean addOpsPolicyGroupHosts(Long policyGroupId, String hostIds) throws OpenapiException;

    /*--------------------------------------------------------------------------------------------------------------------*/

    /**
     * 授权子策略给组织单元
     *
     * @param policyId      子策略ID
     * @param departmentIds 组织单元ID列表
     * @return
     * @throws OpenapiException
     */
    Boolean addOpsPolicyDepartments(Long policyId, String departmentIds) throws OpenapiException;


    /**
     * 修改子策略的条目，目前不支持批量类型
     *
     * @param policyId
     * @param items
     */
    OpsPolicyItemResult updateOpsPolicyItems(Long policyId, Map<String, Object> items) throws OpenapiException;

    /**
     * 授权子策略给团队成员
     *
     * @param policyId  子策略ID
     * @param memberIds 团队成员id列表，逗号分割
     * @return
     * @throws OpenapiException
     */
    Boolean addOpsPolicyMembers(Long policyId, String memberIds) throws OpenapiException;

    /**
     * 授权子策略给角色
     *
     * @param policyId 子策略ID
     * @param roleIds  角色ID列表
     * @return
     * @throws OpenapiException
     */
    Boolean addOpsPolicyRoles(Long policyId, String roleIds) throws OpenapiException;

    /**
     * 修改策略组关联主机
     *
     * @param policyGroupId 策略组ID
     * @param hostIds       主机ID列表，用逗号连接
     * @return
     * @throws OpenapiException
     */
    Boolean updateOpsPolicyGroupHosts(Long policyGroupId, String hostIds) throws OpenapiException;


    /**
     * 修改策略组子策略优先级
     *
     * @param policyGroupId   策略组ID
     * @param sortedPolicyIds 子策略ID排序列表
     * @return
     * @throws OpenapiException
     */
    Boolean updateOpsPolicyGroupPriorities(Long policyGroupId, String sortedPolicyIds) throws OpenapiException;


    /*--------------------------------------------------------------------------------------------------------------------*/

    /**
     * 删除子策略
     *
     * @param policyId
     * @return
     * @throws OpenapiException
     */
    Boolean deleteOpsPolicy(Long policyId) throws OpenapiException;

    /**
     * 删除策略组
     *
     * @param policyGroupId
     * @return
     * @throws OpenapiException
     */
    Boolean deleteOpsPolicyGroup(Long policyGroupId) throws OpenapiException;

    /**
     * 删除策略组关联的主机
     *
     * @param policyGroupId 策略组ID
     * @param hostIds       主机ID列表，用逗号连接
     * @return
     * @throws OpenapiException
     */
    Boolean deleteOpsPolicyGroupHosts(Long policyGroupId, String hostIds) throws OpenapiException;

}
