package com.cloudbility.openapi.client;

import com.cloudbility.openapi.OpenapiClient;
import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.OrderApi;
import com.cloudbility.openapi.common.DateUtil;
import com.cloudbility.openapi.common.JsonUtil;
import com.cloudbility.openapi.response.BooleanResult;
import com.cloudbility.openapi.response.OrderListResult;
import com.cloudbility.openapi.response.PagedOrderListResult;
import com.cloudbility.openapi.vo.OrderVo;
import com.cloudbility.openapi.vo.QuotaVo;
import com.cloudbility.openapi.vo.Scope;
import com.cloudbility.openapi.vo.ServeType;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

/**
 * Created by lidonghao on 2020/8/7.
 */
public class DefaultOrderApi extends AbstractApi implements OrderApi {

    private static final Logger logger = LoggerFactory.getLogger(OrderApi.class);

    public DefaultOrderApi(OpenapiClient client) {
        super(client);
    }

    @Override
    public PagedOrderListResult getOrdersByTeam(Long teamId, ServeType serveType, int page, int size) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.GET)
                .path("/order/byTeam")
                .paramShouldNotEmpty("teamId", teamId)
                .paramIfNotNull("serveType", serveType)
                .paramShouldNotEmpty("page", page)
                .paramShouldNotEmpty("size", size)
                .build(), PagedOrderListResult.class);
    }

    @Override
    public PagedOrderListResult getValidOrdersByTeam(Long teamId, String moduleCode, int page, int size) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.GET)
                .path("/order/valid")
                .paramShouldNotEmpty("teamId", teamId)
                .paramIfNotNull("moduleCode", moduleCode)
                .paramShouldNotEmpty("page", page)
                .paramShouldNotEmpty("size", size)
                .build(), PagedOrderListResult.class);
    }

    @Override
    public OrderVo getTrailOrderByTeam(Long teamId, String moduleCode) throws OpenapiException {
        OrderListResult result = execute(RequestorBuilder.create()
                .method(HttpMethod.GET)
                .path("/order/trial")
                .paramShouldNotEmpty("teamId", teamId)
                .paramShouldNotEmpty("moduleCode", moduleCode)
                .build(), OrderListResult.class);
        List<OrderVo> orders = result.getOrders();
        return orders != null && orders.size() > 0 ? orders.get(0) : null;
    }

    @Override
    public OrderListResult getOrdersByModule(Long teamId, String moduleCode) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.GET)
                .path("/order/byModule")
                .paramShouldNotEmpty("teamId", teamId)
                .paramShouldNotEmpty("moduleCode", moduleCode)
                .build(), OrderListResult.class);
    }

    @Override
    public OrderVo getOrderByNumber(String orderNumber) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.GET)
                .path("/order/byNumber")
                .paramShouldNotEmpty("orderNumber", orderNumber)
                .build(), OrderVo.class);
    }

    @Override
    public OrderVo createOrder(Long userId, Long teamId, ServeType serveType, int productTerm, BigDecimal price,
                               String moduleCode, String moduleName, String editionCode, String editionName,
                               List<QuotaVo> quotas, Date serveExpireTime, String productId, Scope scope) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/order/create")
                .paramShouldNotEmpty("userId", userId)
                .paramShouldNotEmpty("teamId", teamId)
                .paramShouldNotEmpty("serveType", serveType)
                .paramShouldNotEmpty("productTerm", productTerm)
                .paramShouldNotEmpty("price", price)
                .paramIfNotNull("serveType", serveType)
                .paramIfNotNull("moduleCode", moduleCode)
                .paramIfNotNull("moduleName", moduleName)
                .paramIfNotNull("editionCode", editionCode)
                .paramIfNotNull("editionName", editionName)
                .paramIfNotNull("quotas", quotas != null && quotas.size() > 0 ? JsonUtil.toJson(quotas) : null)
                .paramIfNotNull("serveExpireTime", serveExpireTime != null ? DateUtil.formatDate(serveExpireTime) : null)
                .paramIfNotNull("productId", productId)
                .paramIfNotNull("scope", scope)
                .build(), OrderVo.class);
    }

    @Override
    public OrderVo finishOrder(String orderNumber, boolean cancel, String reason) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/order/finish")
                .paramShouldNotEmpty("orderNumber", orderNumber)
                .paramShouldNotEmpty("cancel", cancel)
                .paramIfNotNull("reason", reason)
                .build(), OrderVo.class);
    }

    @Override
    public OrderVo createOrderFromMarket(String teamCode, String moduleCode, String editionCode, int productTerm, ServeType serveType, List<QuotaVo> quotaVos, Scope scope) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/order/createFromMarket")
                .paramShouldNotEmpty("teamCode", teamCode)
                .paramShouldNotEmpty("serveType", serveType)
                .paramShouldNotEmpty("moduleCode", moduleCode)
                .paramShouldNotEmpty("editionCode", editionCode)
                .paramShouldNotEmpty("productTerm", productTerm)
                .paramShouldNotEmpty("scope", scope)
                .paramIfNotNull("quotas", quotaVos != null && quotaVos.size() > 0 ? JsonUtil.toJson(quotaVos) : null)
                .build(), OrderVo.class);
    }

    @Override
    public OrderVo finishMarketOrder(String orderNumber, boolean cancel, String reason) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/order/finish")
                .paramShouldNotEmpty("orderNumber", orderNumber)
                .paramShouldNotEmpty("cancel", cancel)
                .paramIfNotNull("reason", reason)
                .build(), OrderVo.class);
    }

    @Override
    public OrderVo createTrialOrderMarket(Integer teamId, String moduleCode) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/order/createTrialOrder")
                .paramShouldNotEmpty("teamId", teamId)
                .paramShouldNotEmpty("moduleCode", moduleCode)
                .build(), OrderVo.class);
    }

    @Override
    public boolean updateOrderScope(String orderNumber, Scope scope) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/order/updateScope")
                .paramShouldNotEmpty("orderNumber", orderNumber)
                .paramShouldNotEmpty("scope", scope)
                .build(), BooleanResult.class).isSuccess();
    }

}
