package com.cloudbility.openapi.client;

import com.cloudbility.openapi.OpenapiClient;
import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.OtherApi;
import com.cloudbility.openapi.common.StringUtil;
import com.cloudbility.openapi.request.OAuthLoginRequest;
import com.cloudbility.openapi.request.SmsRequest;
import com.cloudbility.openapi.response.*;
import com.cloudbility.openapi.vo.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Date;
import java.util.List;
import java.util.Map;

public class DefaultOtherApi extends AbstractApi implements OtherApi {

    private static final Logger logger = LoggerFactory.getLogger(OtherApi.class);

    public DefaultOtherApi(OpenapiClient client) {
        super(client);
    }

    @Override
    public AkAccessToken getOAuth(String accessKeyId, String accessKeySecret, int expireSeconds) throws OpenapiException {
        if (!StringUtil.startWithIgnoreCase(client.config().getEndpoint(), "https")) {
            logger.warn("requesting oauth token using AK through non https endpoints is unsafe");
        }
        return execute(RequestorBuilder.create()
                .get("/oauth")
                .noAuthorized()
                .paramShouldNotEmpty("accessKeyId", accessKeyId)
                .paramShouldNotEmpty("accessKeySecret", accessKeySecret)
                .paramShouldNotEmpty("expireSeconds", expireSeconds)
                .build(), AkAccessToken.class);
    }

    @Override
    public AkAccessToken getOAuth(int expireSeconds) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/oauthV2")
                .paramShouldNotEmpty("expireSeconds", expireSeconds)
                .build(), AkAccessToken.class);
    }

    @Override
    public CheckTokenResult checkOauth(String token) throws OpenapiException{
        return execute(RequestorBuilder.create()
                .get("/checkOauth")
                .paramShouldNotEmpty("token", token)
                .build(), CheckTokenResult.class);
    }

    @Override
    public void deleteOAuth(String accessKeyId, String accessKeySecret, String token) throws OpenapiException {
        if (!StringUtil.startWithIgnoreCase(client.config().getEndpoint(), "https")) {
            logger.warn("requesting oauth token using AK through non https endpoints is unsafe");
        }
        execute(RequestorBuilder.create()
                .delete("/oauth/" + token)
                .noAuthorized()
                .build(), OpenapiResponse.class);
    }

    @Override
    public void deleteOAuth(String token) throws OpenapiException {
        execute(RequestorBuilder.create()
                .delete("/oauthV2/" + token)
                .build(), OpenapiResponse.class);
    }

    @Override
    public boolean ping() throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/ping"),
                BooleanResult.class).isSuccess();
    }

    @Override
    public GetQuotaResult getQuota() throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/quota")
                .build(), GetQuotaResult.class);
    }

    @Override
    public List<PermissionQuota> getPermissionQuota(List<Permission> permissions, Date time) throws OpenapiException {
        PermissionQuotaResult result = execute(RequestorBuilder.create()
                .method(HttpMethod.GET)
                .path("/permissionQuota")
                .param("permissions", StringUtil.join(permissions, ","))
                .build(), PermissionQuotaResult.class);
        return result.getQuotaList();
    }

    @Override
    public SmsResult sendSms(SmsTemplate template, List<String> phones, List<String> parameters) throws OpenapiException {
        SmsRequest request = new SmsRequest();
        request.setTemplate(template);
        request.setParameters(parameters);
        request.setPhones(phones);
        return execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/sms")
                .body(request)
                .build(), SmsResult.class);
    }


    @Override
    public Map<String, SmsResult> sendSmsV2(SmsTemplate template, List<String> phones, List<String> parameters) throws OpenapiException {
        SmsRequest request = new SmsRequest();
        request.setTemplate(template);
        request.setParameters(parameters);
        request.setPhones(phones);
        SmsResultV2 resultV2 = execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/smsV2")
                .body(request)
                .build(), SmsResultV2.class);
        return resultV2.getResultMap();
    }

    @Override
    public SmsResult sendSmsFromOtherProvider(SmsTemplate template, List<String> phones, List<String> parameters) throws OpenapiException {
        SmsRequest request = new SmsRequest();
        request.setTemplate(template);
        request.setParameters(parameters);
        request.setPhones(phones);
        return execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .body(request)
                .build(), SmsResult.class);
    }

    @Override
    public OAuthUrlResult getOAuthLogin(OAuthLoginRequest request) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.GET)
                .path("/oauthLogin")
                .paramShouldNotEmpty("userId", request.getUserId())
                .paramShouldNotEmpty("oneoff", request.isOneoff())
                .paramShouldNotEmpty("expireSeconds", request.getExpireSeconds())
                .paramIfNotNull("teamId", request.getTeamId())
                .paramIfNotNull("hostId", request.getHostId())
                .paramIfNotNull("page", request.getPage())
                .paramIfNotNull("redirectUrl", request.getRedirectUrl())
                .paramIfNotNull("workflowInstanceId", request.getWorkflowInstanceId())
                .build(), OAuthUrlResult.class);
    }

    @Override
    public CheckJwtResult checkJwt(String jwt) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .post("/checkJwt")
                .paramShouldNotEmpty("jwt", jwt)
                .build(), CheckJwtResult.class);
    }

    @Override
    public boolean isSaaS() throws OpenapiException {
        BooleanResponse response = execute(RequestorBuilder.create()
                .get("/checkSaaS")
                .build(), BooleanResponse.class);
        return response.getValue();
    }

    @Override
    public boolean verifyPhoneCode(String sessionId, String code) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.GET)
                .path("/verifyPhoneCode/" + sessionId + "/" + code)
                .build(), BooleanResult.class).isSuccess();
    }

}
