package com.cloudbility.openapi.client;

import com.cloudbility.openapi.OpenapiClient;
import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.ScriptApi;
import com.cloudbility.openapi.common.Base64Util;
import com.cloudbility.openapi.common.StringUtil;
import com.cloudbility.openapi.response.OpenapiResponse;
import com.cloudbility.openapi.vo.*;

import java.io.*;
import java.net.URLEncoder;
import java.util.List;

public class DefaultScriptApi extends AbstractApi implements ScriptApi {

    public DefaultScriptApi(OpenapiClient client) {
        super(client);
    }

    @Override
    public ScriptLibraryListResult getScriptLibraryList(long teamId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/script/library")
                        .paramShouldNotEmpty("teamId", teamId)
                        .build(),
                ScriptLibraryListResult.class);
    }

    @Override
    public void updateScriptLibrary(long libraryId, String name) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .put("/script/library/" + libraryId)
                        .paramShouldNotEmpty("name", name),
                OpenapiResponse.class);
    }

    @Override
    public ScriptLibraryVo createScriptLibrary(long teamId, String name, String description, Long creator) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .post("/script/library")
                        .paramShouldNotEmpty("name", name)
                        .paramShouldNotEmpty("teamId", teamId)
                        .paramIfNotNull("description", description)
                        .paramIfNotNull("creator", creator),
                ScriptLibraryVo.class);
    }

    @Override
    public ScriptLibraryVo getScriptLibrary(long id) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/script/library/" + id)
                        .build(),
                ScriptLibraryVo.class);
    }

    public ScriptLibraryVo getScriptLibraryByName(long teamId, String name) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/script/library/byName")
                        .paramShouldNotEmpty("teamId", teamId)
                        .paramShouldNotEmpty("name", name)
                        .build(),
                ScriptLibraryVo.class);
    }

    @Override
    public void deleteScriptLibrary(long libraryId) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .delete("/script/library/" + libraryId),
                OpenapiResponse.class);
    }

    @Override
    public int deleteScriptsByLibrary(long libraryId) throws OpenapiException {
        CountResult result = execute(RequestorBuilder.create()
                        .delete("/script/byLibrary/" + libraryId),
                CountResult.class);
        return result.getCount();
    }

    @Override
    public ScriptListResult getScriptList(long libraryId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/script/byLibrary/" + libraryId),
                ScriptListResult.class);
    }

    @Override
    public ScriptVo createScript(long libraryId, String name, String content, String parameter, String description, ScriptLanguage language, Integer timeout, Long creator) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .post("/script")
                        .paramShouldNotEmpty("libraryId", libraryId)
                        .paramShouldNotEmpty("name", name)
                        .paramShouldNotEmpty("content", content)
                        .paramShouldNotEmpty("language", language)
                        .paramIfNotNull("parameter", parameter)
                        .paramIfNotNull("description", description)
                        .paramIfNotNull("creator", creator)
                        .paramIfNotNull("timeout", timeout),
                ScriptVo.class);
    }

    @Override
    public ScriptVo getScript(long id) throws OpenapiException {
        return getScript(id, false);
    }

    @Override
    public ScriptVo getScript(long id, boolean includeContent) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/script/" + id)
                        .paramIfNotNull("includeContent", includeContent),
                ScriptVo.class);
    }

    @Override
    public ScriptVo getScriptByName(long libraryId, String name, boolean includeContent) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/script/byName")
                        .paramShouldNotEmpty("libraryId", libraryId)
                        .paramShouldNotEmpty("name", name)
                        .paramIfNotNull("includeContent", includeContent)
                ,
                ScriptVo.class);
    }

    @Override
    public void deleteScript(long id) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .delete("/script/" + id),
                OpenapiResponse.class);
    }

    @Override
    public void updateScriptInfo(long id, String name, int timeout, String description) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .put("/script/" + id)
                        .paramShouldNotEmpty("name", name)
                        .paramShouldNotEmpty("timeout", timeout)
                        .paramIfNotNull("description", description),
                OpenapiResponse.class);
    }

    @Override
    public void updateScriptContent(long id, String content) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .put("/script/" + id + "/content")
                        .paramShouldNotEmpty("content", content),
                OpenapiResponse.class);
    }

    @Override
    public void updateScriptContentWithPayload(long id, InputStream inputStream, long length) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .put("/script/" + id + "/contentWithPayload")
                        .body(inputStream)
                        .contentLength(length),
                OpenapiResponse.class);
    }

    @Override
    public void updateScriptContentWithPayload(long id, File file) throws OpenapiException, FileNotFoundException {
        InputStream inputStream = null;
        try {
            inputStream = new FileInputStream(file);
            updateScriptContentWithPayload(id, inputStream, file.length());
        } finally {
            if (inputStream != null) {
                try {
                    inputStream.close();
                } catch (IOException e) {
                }
            }
        }
    }

    @Override
    public JobVo executeScript(int scriptId, ScriptExecutingHostsInfo executingHostsInfo) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .post("/script/" + scriptId + "/execute")
                        .body(executingHostsInfo),
                JobVo.class);
    }

    @Override
    public ScriptRunResult getScriptRunResult(long jobId, boolean includeOutput) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/script/job/" + jobId)
                        .paramIfNotNull("includeOutput", includeOutput),
                ScriptRunResult.class);
    }

    @Override
    public InputStreamResult getScriptOutputOnHost(long jobId, long hostId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/script/jobOutput/" + jobId + "/" + hostId), InputStreamResult.class);
    }

    @Override
    public ScriptLibraryListResult getPreInstalledScriptLibraries(long teamId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/preInstalledScriptLibraries")
                        .paramShouldNotEmpty("teamId", teamId),
                ScriptLibraryListResult.class);
    }

    @Override
    public ScriptLibraryListResult getPreInstalledSubLibrary(long teamId, long libraryId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/preInstalledLibrary/" + libraryId + "/subLibrary")
                        .paramShouldNotEmpty("teamId", teamId),
                ScriptLibraryListResult.class);
    }

    @Override
    public ScriptListResult getScriptsByPreInstalledLibrary(long libraryId, boolean includeContent, int page, int size) throws OpenapiException {
        return execute(RequestorBuilder.create()
                        .get("/script/byPreInstalledLibrary/" + libraryId)
                        .paramShouldNotEmpty("includeContent", includeContent)
                        .paramShouldNotEmpty("page", page)
                        .paramShouldNotEmpty("size", size),
                ScriptListResult.class);
    }

    @Override
    public JobVo executeCommandWithAgent(List<Long> hostIds, String platform, String command) throws OpenapiException{
        //先Base64加密
        try{
            command = Base64Util.encodeURLSafe(command.getBytes("UTF-8"));
        }catch (Exception e){
            //ignore
        }
        return execute(RequestorBuilder.create()
                        .post("/script/executeCommandWithAgent")
                        .paramShouldNotEmpty("hostIds", hostIds != null && hostIds.size() > 0 ? StringUtil.join(hostIds, ",") : null)
                        .paramShouldNotEmpty("platform", platform)
                        .paramShouldNotEmpty("command", command),
                JobVo.class);
    }


}
