package com.cloudbility.openapi.client;

import com.cloudbility.openapi.OpenapiClient;
import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.TeamApi;
import com.cloudbility.openapi.common.StringUtil;
import com.cloudbility.openapi.request.ImportTeamRequest;
import com.cloudbility.openapi.response.*;
import com.cloudbility.openapi.vo.CountResult;
import com.cloudbility.openapi.vo.ImportTeamVo;

import java.util.List;

public class DefaultTeamApi extends AbstractApi implements TeamApi {

    public DefaultTeamApi(OpenapiClient client) {
        super(client);
    }

    @Override
    public ImportTeamResult importTeams(List<ImportTeamVo> teams) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/team")
                .body(new ImportTeamRequest(teams))
                .build(), ImportTeamResult.class);
    }

    @Override
    public void addMembersByCode(String teamCode, String account) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .post("/team/byCode/" + teamCode + "/members/" + account).build(),
                Void.class);
    }

    @Override
    public void addMembers(long teamId, List<Long> userIds) throws OpenapiException {
        execute(RequestorBuilder.create()
                        .post("/team/" + teamId + "/members/" + StringUtil.join(userIds, ",")).build(),
                Void.class);
    }

    @Override
    public GetTeamResult getTeamByCode(String code, boolean includeMembers) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/team/byCode/" + code)
                .paramIfNotNull("includeMembers", includeMembers)
                .build(), GetTeamResult.class);
    }

    @Override
    public GetTeamResult getTeam(long teamId, boolean includeMembers) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/team/" + teamId)
                .paramIfNotNull("includeMembers", includeMembers)
                .build(), GetTeamResult.class);
    }

    @Override
    public PagedUserListResult getTeamMembersByCode(String teamCode) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/team/byCode/" + teamCode + "/members")
                .build(), PagedUserListResult.class);
    }

    @Override
    public PagedUserListResult getTeamMembers(long teamId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/team/" + teamId + "/members")
                .build(), PagedUserListResult.class);
    }

    @Override
    public GetTeamQuotaResult getTeamQuotaByCode(String teamCode) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/team/byCode/" + teamCode + "/quota")
                .build(), GetTeamQuotaResult.class);
    }

    @Override
    public GetTeamQuotaResult getTeamQuota(long teamId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/team/" + teamId + "/quota")
                .build(), GetTeamQuotaResult.class);
    }

    @Override
    public GetTeamListResult getTeams(Integer page, Integer size) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/teams")
                .paramIfNotNull("page", page)
                .paramIfNotNull("size", size)
                .build(), GetTeamListResult.class);
    }

    @Override
    public TeamBriefResult findTeamIds() throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/team/findIds")
                .build(), TeamBriefResult.class);
    }

    @Override
    public TeamListResult findByName(String name) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/teams/byName")
                .paramShouldNotEmpty("name", name)
                .build(), TeamListResult.class);
    }

    @Override
    public TeamListResult findByNameLike(String name) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/teams/byNameLike")
                .paramShouldNotEmpty("name", name)
                .build(), TeamListResult.class);
    }

    @Override
    public void updateTeamNameOrCreatorByCode(String code, String name, String creatorAccount) throws OpenapiException {
        execute(RequestorBuilder.create()
                .method(HttpMethod.PUT)
                .path("/team/byCode/" + code)
                .paramIfNotNull("name", name)
                .paramIfNotNull("creatorAccount", creatorAccount)
                .build(), Void.class);

    }

    @Override
    public void updateTeamNameOrCreator(long teamId, String name, Long creator) throws OpenapiException {
        execute(RequestorBuilder.create()
                .method(HttpMethod.PUT)
                .path("/team/" + teamId)
                .paramIfNotNull("name", name)
                .paramIfNotNull("creator", creator)
                .build(), Void.class);
    }

    @Override
    public void updateTeamQuota(long teamId, Integer members, Integer hosts, Integer cdns, Integer buckets, Integer dbs) throws OpenapiException {
        execute(RequestorBuilder.create()
                .method(HttpMethod.PUT)
                .path("/team/" + teamId + "/quota")
                .paramIfNotNull("hosts", hosts)
                .paramIfNotNull("cdns", cdns)
                .paramIfNotNull("buckets", buckets)
                .paramIfNotNull("dbs", dbs)
                .build(), Void.class);
    }

    @Override
    public void updateTeamQuotaByCode(String teamCode, Integer members, Integer hosts, Integer cdns, Integer buckets, Integer dbs) throws OpenapiException {
        execute(RequestorBuilder.create()
                .method(HttpMethod.PUT)
                .path("/team/byCode/" + teamCode + "/quota")
                .paramIfNotNull("hosts", hosts)
                .paramIfNotNull("cdns", cdns)
                .paramIfNotNull("buckets", buckets)
                .paramIfNotNull("dbs", dbs)
                .build(), Void.class);
    }

    @Override
    public void updateTeamCode(Long teamId, String code) throws OpenapiException {
        execute(RequestorBuilder.create()
                .put("/team/" + teamId + "/code")
                .paramShouldNotEmpty("code", code)
                .build(), Void.class);
    }

    @Override
    public void deleteTeams(List<Long> teamIds) throws OpenapiException {
        if (teamIds == null || teamIds.size() == 0) {
            throw new IllegalArgumentException("teamIds is empty");
        }
        execute(RequestorBuilder.create()
                .method(HttpMethod.DELETE)
                .path("/team/" + StringUtil.join(teamIds, ","))
                .build(), Void.class);
    }

    @Override
    public void deleteTeamByCode(String code) throws OpenapiException {
        execute(RequestorBuilder.create()
                .method(HttpMethod.DELETE)
                .path("/team/byCode/" + code)
                .build(), Void.class);
    }

    @Override
    public void deleteTeamMembers(Long teamId, List<Long> memberIds) throws OpenapiException {
        execute(RequestorBuilder.create()
                .delete("/team/" + teamId + "/members/" + StringUtil.join(memberIds, ","))
                .build(), Void.class);
    }

    @Override
    public void deleteTeamMemberByCode(String teamCode, String memberAccount) throws OpenapiException {
        execute(RequestorBuilder.create()
                .method(HttpMethod.DELETE)
                .path("/team/byCode/" + teamCode + "/members/" + memberAccount)
                .build(), Void.class);
    }

    @Override
    public QuotaItemResult getQuotaItems() throws OpenapiException {
        return execute(RequestorBuilder.create()
                .get("/quotaItems")
                .build(), QuotaItemResult.class);
    }
    @Override
    public int countTeamMemberOnline(long teamId) throws OpenapiException {
        CountResult result = execute(RequestorBuilder.create()
                .get("/team/" + teamId + "/countOnlineMembers")
                .build(), CountResult.class);
        return result.getCount();
    }

    @Override
    public GetTeamResult createTeam(String teamName, String teamCode, Long creatorId) throws OpenapiException {
        return execute(RequestorBuilder.create()
                .method(HttpMethod.POST)
                .path("/team/createTeam")
                .paramShouldNotEmpty("teamName", teamName)
                .paramShouldNotEmpty("teamCode", teamCode)
                .paramShouldNotEmpty("creatorId", creatorId)
                .build(), GetTeamResult.class);
    }
}
