package com.cloudbility.openapi.client;

import com.cloudbility.openapi.OpenapiErrorCode;
import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.common.*;
import com.cloudbility.openapi.common.networking.HttpClientBuilder;
import com.cloudbility.openapi.requestor.OpenapiRequestor;
import com.cloudbility.openapi.vo.InputStreamResult;
import org.apache.http.Consts;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.*;
import org.apache.http.entity.BasicHttpEntity;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.mime.HttpMultipartMode;
import org.apache.http.entity.mime.MultipartEntityBuilder;
import org.apache.http.entity.mime.content.ContentBody;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.*;
import java.nio.charset.Charset;
import java.text.SimpleDateFormat;
import java.util.*;

public class HttpComponentsOpenapiExecutor implements OpenapiExecutor {
    private static final Logger log = LoggerFactory.getLogger(HttpComponentsOpenapiExecutor.class);
    private HttpClient httpClient;
    private OpenapiConfig openapiConfig;
    protected String timestampPattern = "yyyy-MM-dd'T'HH:mm:ss'Z'";

    public HttpComponentsOpenapiExecutor(OpenapiConfig openapiConfig, HttpClient httpClient) {
        this.openapiConfig = openapiConfig;
        this.httpClient = httpClient;
    }

    public HttpComponentsOpenapiExecutor(OpenapiConfig openapiConfig) {
        this(openapiConfig, HttpClientBuilder.buildHttpClient(openapiConfig));
    }

    protected String buildUrl(OpenapiRequestor request) throws OpenapiException {
        RequestBuilder builder = RequestBuilder.get(StringUtil.joinPath(openapiConfig.getEndpoint(), request.getPath()));
        boolean needSign = true;
        if (request.isTokenSet() || !request.needAuthorized()) {
            needSign = false;
        }
        Map<String, Object> signMap = new HashMap<>();
        if (needSign) {
            SimpleDateFormat sdf = new SimpleDateFormat(timestampPattern);
            signMap.put("timestamp", sdf.format(new Date()));
            signMap.put("version", openapiConfig.getVersion());
            signMap.put("nonce", StringUtil.randomPasswordStr(6));
            signMap.put("accessKeyId", openapiConfig.getAccessKeyId());
        }
        Map<String, Object> mkv = request.getQueryParameters();
        if (mkv != null) {
            for (Map.Entry<String, Object> entry : mkv.entrySet()) {
                if (signMap.containsKey(entry.getKey())) {
                    throw new OpenapiException(OpenapiErrorCode.InvalidParameter("参数" + entry.getKey() + "重复"));
                }
                Object value = entry.getValue();
                if (request.isMultipart() && value instanceof MultipartPart) {
                    continue;
                }
                signMap.put(entry.getKey(), value);
            }
        }
        if (needSign) {
            if (StringUtil.isEmpty(openapiConfig.getAccessKeySecret())) {
                throw new IllegalArgumentException("cannot sign,accessKeySecret not set");
            }
            String signature = SignUtil.sign(request.getPath(), signMap, request.getHttpMethod().toString(), openapiConfig.getAccessKeySecret());
            signMap.put("signature", signature);
        }
        for (Map.Entry<String, Object> entry : signMap.entrySet()) {
            Object value = entry.getValue();
            if (value instanceof List) {
                for (Object item : (List) value) {
                    builder.addParameter(entry.getKey(), String.valueOf(item));
                }
            } else {
                builder.addParameter(entry.getKey(), String.valueOf(value));
            }
        }
        return builder.build().getURI().toString();
    }

    @Override
    public <R> R execute(OpenapiRequestor request, Class<R> responseClazz) throws OpenapiException {
        String url = buildUrl(request);
        HttpRequestBase hur = null;
        boolean stream = false;
        try {
            hur = createHttpUriRequest(request, url);
            HttpResponse response = httpClient.execute(hur);
            int statusCode = response.getStatusLine().getStatusCode();
            HttpEntity entity = response.getEntity();
            Map<String, Object> map = null;
            OpenapiErrorCode errorCode = null;
            stream = InputStreamResult.class.equals(responseClazz);
            boolean ok = statusCode >= 200 && statusCode < 300;
            if (!stream || !ok) {
                try {
                    map = JsonUtil.fromJsonAsMap(new InputStreamReader(entity.getContent(), "UTF-8"));
                    if (map == null) {
                        map = Collections.emptyMap();
                    }
                    errorCode = OpenapiErrorCode.fromRequestResult(map);
                } catch (Throwable throwable) {
                    log.error("invalid response:not a json :{}", response.getStatusLine(), throwable);
                    errorCode = OpenapiErrorCode.UnknownError(response.getStatusLine().toString());
                }
            }
            if (errorCode != null) {
                String requestId = map != null ? (String) map.get("requestId") : null;
                if (requestId != null) {
                    throw new OpenapiException(statusCode, requestId, errorCode);
                } else {
                    throw new OpenapiException(statusCode, errorCode);
                }
            }
            if (ok) {
                if (stream) {
                    HttpRequestBase requestBase = hur;
                    InputStream s = new CloseAwareInputStream(response.getEntity().getContent(), () -> {
                        if (requestBase != null) {
                            requestBase.releaseConnection();
                        }
                    });
                    return (R) (new InputStreamResult(s, response.getEntity().getContentLength()));
                }
                return JsonUtil.fromJsonMap(map, responseClazz);
            } else {
                throw new OpenapiException(statusCode, OpenapiErrorCode.UnknownError("Server response:" + response.getStatusLine()));
            }
        } catch (Throwable e) {
            if (e instanceof OpenapiException) {
                throw (OpenapiException) e;
            }
            throw new OpenapiException(OpenapiErrorCode.UnknownError("Error:" + e.getMessage()), e);
        } finally {
            if (hur != null && !stream) {
                hur.releaseConnection();
            }
        }
    }

    private HttpRequestBase createHttpUriRequest(OpenapiRequestor request, String url) throws IOException {
        HttpRequestBase httpReq = null;
        switch (request.getHttpMethod()) {
            case GET:
                httpReq = new HttpGet(url);
                break;
            case PUT:
                httpReq = new HttpPut(url);
                break;
            case DELETE:
                httpReq = new HttpDelete(url);
                break;
            case POST:
                httpReq = new HttpPost(url);
                break;
            default:
                throw new IllegalArgumentException(request.getHttpMethod() + " not supported");
        }
        //默认两个header
        httpReq.setHeader("Content-type", "application/json; charset=UTF-8");
        httpReq.setHeader("Accept", "application/json");
        if (openapiConfig.getSharedHeaders() != null) {
            for (Map.Entry<String, String> headerEntry : openapiConfig.getSharedHeaders().entrySet()) {
                httpReq.setHeader(headerEntry.getKey(), headerEntry.getValue());
            }
        }
        if (request.getHeaders() != null) {
            for (Map.Entry<String, String> headerEntry : request.getHeaders().entrySet()) {
                httpReq.setHeader(headerEntry.getKey(), headerEntry.getValue());
            }
        }
        try {
            if (request.isMultipart()) {//表单上传
                httpReq.removeHeaders("Content-type");//会自动生成，不能设置
                if (!(httpReq instanceof HttpPost) && !(httpReq instanceof HttpPut)) {
                    throw new IllegalArgumentException(request.getHttpMethod() + " with multipart/form-data not supported");
                }
                HttpEntityEnclosingRequestBase hrb = (HttpEntityEnclosingRequestBase) httpReq;
                MultipartEntityBuilder builder = MultipartEntityBuilder.create();
                for (Map.Entry<String, Object> entry : request.getQueryParameters().entrySet()) {
                    String key = entry.getKey();
                    Object value = entry.getValue();
                    builder.setCharset(Consts.UTF_8);
                    builder.setMode(HttpMultipartMode.BROWSER_COMPATIBLE);
                    if (value instanceof String) {
                        builder.addTextBody(key, (String) value);
                    } else if (value instanceof MultipartPart) {
                        MultipartPart p = (MultipartPart) value;
                        String theContentType = p.getContentType();
                        if (theContentType == null) {
                            theContentType = p.getFilename() != null ? MimetypesUtil.getContentType(p.getFilename()) : p.getContentType();
                        }
                        ContentType contentType = theContentType != null ? ContentType.parse(theContentType) : ContentType.DEFAULT_BINARY;
                        if (p.getContentType() != null) {
                            contentType = ContentType.parse(p.getContentType());
                        }
                        if (p.getBodyPart() instanceof String) {
                            builder.addTextBody(p.getName(), (String) p.getBodyPart());
                        } else if (p.getBodyPart() instanceof byte[]) {
                            builder.addBinaryBody(p.getName(), (byte[]) p.getBodyPart(), contentType, p.getFilename());
                        } else if (p.getBodyPart() instanceof InputStream) {
                            builder.addBinaryBody(p.getName(), (InputStream) p.getBodyPart(), contentType, p.getFilename());
                        } else if (p.getBodyPart() instanceof File) {
                            builder.addBinaryBody(p.getName(), (File) p.getBodyPart(), contentType, p.getFilename());
                        } else if (p.getBodyPart() instanceof ContentBody) {
                            builder.addPart(p.getName(), (ContentBody) p.getBodyPart());
                        }
                    }
                }
                hrb.setEntity(builder.build());
            } else {
                BasicHttpEntity httpEntity = new BasicHttpEntity();
                Object req = request.getRequest();
                if (req != null) {
                    if (!(httpReq instanceof HttpEntityEnclosingRequestBase)) {
                        throw new IllegalArgumentException(request.getHttpMethod() + " should not have request body");
                    }
                    HttpEntityEnclosingRequestBase requestBase = (HttpEntityEnclosingRequestBase) httpReq;
                    if (req instanceof InputStream) {
                        InputStream inputStream = (InputStream) req;
                        httpEntity.setContent(inputStream);
                        if (request.getContentLength() > 0) {
                            httpEntity.setContentLength(request.getContentLength());
                        } else {
                            //TODO 这个方法不靠谱
                            httpEntity.setContentLength(inputStream.available());
                        }
                    } else {
                        NoCopyByteArrayOutputStream outputStream = new NoCopyByteArrayOutputStream(256);
                        JsonUtil.toJson(req, new OutputStreamWriter(outputStream, "UTF-8"));
                        httpEntity.setContent(outputStream.asInputStream());
                        httpEntity.setContentLength(outputStream.size());
                    }
                    requestBase.setEntity(httpEntity);
                }
            }
        } catch (IOException e) {
            throw e;
        }
        return httpReq;
    }

}
