package com.cloudbility.openapi.client;

import com.cloudbility.openapi.common.StringUtil;
import com.cloudbility.openapi.requestor.OpenapiRequestor;
import org.apache.http.entity.mime.content.ContentBody;

import java.io.File;
import java.io.InputStream;
import java.util.*;

public class RequestorBuilder implements OpenapiRequestor {
    private Object requestBody;
    private String path = "/";
    private HttpMethod method;
    private Map<String, String> headers = new HashMap<>(0);

    private Map<String, Object> params = new LinkedHashMap<>();

    private boolean tokenSet;

    private boolean needAuthorized = true;

    private long contentLength = -1;

    private boolean multipart;


    public static RequestorBuilder create() {
        return new RequestorBuilder();
    }

    public RequestorBuilder enableMultipart() {
        this.multipart = true;
        return this;
    }

    public RequestorBuilder path(String path) {
        this.path = path;
        return this;
    }

    public RequestorBuilder post(String path) {
        this.path = path;
        this.method = HttpMethod.POST;
        return this;
    }

    public RequestorBuilder get(String path) {
        this.path = path;
        this.method = HttpMethod.GET;
        return this;
    }

    public RequestorBuilder delete(String path) {
        this.path = path;
        this.method = HttpMethod.DELETE;
        return this;
    }

    public RequestorBuilder noAuthorized() {
        this.needAuthorized = false;
        return this;
    }

    public RequestorBuilder put(String path) {
        this.path = path;
        this.method = HttpMethod.PUT;
        return this;
    }


    public RequestorBuilder method(HttpMethod method) {
        this.method = method;
        return this;
    }

    public RequestorBuilder body(Object requestBody) {
        this.requestBody = requestBody;
        return this;
    }

    public RequestorBuilder contentLength(long length) {
        this.contentLength = length;
        return this;
    }

    public RequestorBuilder header(String name, String value) {
        this.headers.put(name, value);
        return this;
    }

    public RequestorBuilder param(String name, Object value) {
        if (value == null) {
            return this;
        }
        if (!multipart && value instanceof MultipartPart) {
            throw new IllegalArgumentException("multipart not supported");
        }
        if (!(value instanceof MultipartPart)) {
            value = String.valueOf(value);
        }
        Object existsValue = this.params.get(name);
        if (existsValue != null) {
            if (existsValue instanceof List) {
                ((List) existsValue).add(value);
            } else {
                List<Object> list = new ArrayList<>();
                list.add(existsValue);
                list.add(value);
                this.params.put(name, list);
            }
        } else {
            this.params.put(name, value);
        }
        return this;
    }

    public RequestorBuilder token(String token) {
        if (token != null) {
            this.tokenSet = true;
            header("Authorization", token);
        }
        return this;
    }

    @Override
    public boolean isTokenSet() {
        return tokenSet;
    }

    private Class[] supportedMultipartClasses = new Class[]{String.class, byte[].class, File.class, InputStream.class, ContentBody.class};


    public RequestorBuilder multipart(String name, ContentBody inputStream) {
        MultipartPart part = new MultipartPart();
        part.setName(name);
        part.setBodyPart(inputStream);
        return multipart(part);
    }

    public RequestorBuilder multipart(String name, String filename, InputStream inputStream) {
        MultipartPart part = new MultipartPart();
        part.setName(name);
        part.setFilename(filename);
        part.setBodyPart(inputStream);
        return multipart(part);
    }

    public RequestorBuilder multipart(String name, String filename, byte[] file) {
        MultipartPart part = new MultipartPart();
        part.setName(name);
        part.setFilename(filename);
        part.setBodyPart(file);
        return multipart(part);
    }

    public RequestorBuilder multipart(String name, File file) {
        return multipart(name, file.getName(), file);
    }

    public RequestorBuilder multipart(String name, String filename, File file) {
        MultipartPart part = new MultipartPart();
        part.setName(name);
        part.setFilename(filename);
        part.setBodyPart(file);
        return multipart(part);
    }

    public RequestorBuilder multipart(MultipartPart part) {
        if (!multipart) {
            throw new IllegalArgumentException("multipart not supported");
        }
        if (part == null) {
            throw new IllegalArgumentException("part is null");
        }
        if (part.getName() == null) {
            throw new IllegalArgumentException("multipart.name is null");
        }
        if (part.getBodyPart() == null) {
            throw new IllegalArgumentException("bodyPart is null");
        }
        Class clazz = null;
        for (Class supportedMultipartClass : supportedMultipartClasses) {
            if (supportedMultipartClass.isAssignableFrom(part.getBodyPart().getClass())) {
                clazz = supportedMultipartClass;
                break;
            }
        }
        if (clazz == null) {
            throw new IllegalArgumentException("bodyPart class " + part.getBodyPart().getClass() + " not supported");
        }
        param(part.getName(), part);
        return this;
    }

    public RequestorBuilder paramIfNotNull(String name, Object value) {
        if (value != null) {
            if (value instanceof MultipartPart) {
                if (!multipart) {
                    throw new IllegalArgumentException("multipart not supported");
                }
            } else {
                return param(name, value);
            }
        }
        return this;
    }

    public RequestorBuilder paramMuiltiValueIfNotNull(String name, Object... values) {
        if (values != null) {
            for (Object value : values) {
                paramIfNotNull(name, value);
            }
        }
        return this;
    }

    public RequestorBuilder paramMuiltiValueIfNotNull(String name, String... values) {
        if (values != null) {
            for (Object value : values) {
                paramIfNotNull(name, value);
            }
        }
        return this;
    }


    public RequestorBuilder paramShouldNotEmpty(String name, Object value) {
        if (value == null) {
            throw new IllegalArgumentException("param " + name + " must not be empty");
        }
        if ((value instanceof String) && StringUtil.isEmpty((String) value)) {
            throw new IllegalArgumentException("param " + name + " must not be empty");
        }
        return param(name, value);
    }

    public RequestorBuilder headers(Map<String, String> headers) {
        this.headers.putAll(headers);
        return this;
    }

    public RequestorBuilder queryParams(Map<String, String> params) {
        this.params.putAll(params);
        return this;
    }

    public RequestorBuilder build() {
        validate();
        return this;
    }

    private void validate() {
        if (path == null) {
            throw new IllegalArgumentException("path is null");
        }
        if (method == null) {
            throw new IllegalArgumentException("method is null");
        }
    }

    @Override
    public HttpMethod getHttpMethod() {
        return method;
    }

    @Override
    public String getPath() {
        return path;
    }

    @Override
    public Object getRequest() {
        return requestBody;
    }

    @Override
    public Map<String, String> getHeaders() {
        return headers;
    }

    @Override
    public Map<String, Object> getQueryParameters() {
        return params;
    }

    @Override
    public boolean needAuthorized() {
        return needAuthorized;
    }

    @Override
    public long getContentLength() {
        return contentLength;
    }

    @Override
    public boolean isMultipart() {
        return multipart;
    }

    public void setMultipart(boolean multipart) {
        this.multipart = multipart;
    }
}
