package com.cloudbility.openapi.demo.batch1;

import com.cloudbility.openapi.DemoException;
import com.cloudbility.openapi.OpenapiClient;
import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.common.StringUtil;
import com.cloudbility.openapi.request.ImportHostRequest;
import com.cloudbility.openapi.request.ImportUserRequest;
import com.cloudbility.openapi.response.CloudHostResult;
import com.cloudbility.openapi.response.ImportHostResult;
import com.cloudbility.openapi.response.PagedUserListResult;
import com.cloudbility.openapi.response.UserListResult;
import com.cloudbility.openapi.vo.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

public class BatchImporter {
    private Logger logger = LoggerFactory.getLogger(BatchImporter.class);
    private OpenapiClient client;
    //nickname->UserVo
    private Map<String, UserVo> userVoMap = new HashMap<>();
    //vgName->vgVo
    private Map<String, VirtualGroupVo> virtualGroupVoMap = new HashMap<>();
    //ip->hostVo
    private Map<String, CloudHostVo> hostVoMap = new HashMap<>();

    public BatchImporter(OpenapiClient client) {
        this.client = client;
    }

    /**
     * 批量将主机、分组、用户导入到特定团队和云账号中
     *
     * @param teamId
     * @param cloudId
     * @param hosts
     * @throws Exception
     */
    public void importHostAndUser(long teamId, long cloudId, List<HostVgVo> hosts) throws Exception {
        ImportUserRequest request = new ImportUserRequest();
        //1，首先创建用户并加入团队
        for (HostVgVo hostVgVo : hosts) {
            for (String userNickname : hostVgVo.getNicknames()) {
                UserListResult listResult = client.userApi().getUserByNickname(userNickname);
                if (listResult.getUsers().size() > 1) {
                    throw new Exception("昵称为：" + userNickname + "的用户存在多个");
                }
                if (listResult.getUsers().size() == 0) {
                    ImportUserVo userVo = new ImportUserVo();
                    userVo.setJoinTeam(teamId);
                    userVo.setName(userNickname);
                    request.getUsers().add(userVo);
                } else {
                    userVoMap.put(userNickname, listResult.getUsers().get(0));
                }
            }
            VirtualGroupVo groupVo1 = getOrCreateVg(teamId, null, hostVgVo.getVgL1());
            VirtualGroupVo groupVo2 = getOrCreateVg(teamId, groupVo1.getId(), hostVgVo.getVgL2());
            VirtualGroupVo groupVo3 = getOrCreateVg(teamId, groupVo2.getId(), hostVgVo.getVgL3());
        }
        if (request.getUsers().size() > 0) {
            client.userApi().importUsers(request.getUsers());
        }
        PagedUserListResult result = client.userApi().getUsersByTeam(teamId, 1, 10000);
        for (UserVo userVo : result.getUsers()) {
            userVoMap.put(userVo.getNickname(), userVo);
        }
        //vgId=>memberIds
        Map<Long, List<Long>> groupMemberMap = new HashMap<>();
        for (HostVgVo hostVgVo : hosts) {
            VirtualGroupVo groupVo = virtualGroupVoMap.get(hostVgVo.getVgL3());
            List<Long> userIds = hostVgVo.getNicknames().stream().map(name -> userVoMap.get(name).getId()).collect(Collectors.toList());
            List<Long> list = groupMemberMap.get(groupVo.getId());
            if (list == null) {
                list = new ArrayList<>();
            }
            list.addAll(userIds);
            groupMemberMap.put(groupVo.getId(), list);
            if (userIds.size() == 0) {
                logger.error("三级分组:" + hostVgVo.getVgL3() + "没有访问人员");
            }
        }
        for (Map.Entry<Long, List<Long>> entry : groupMemberMap.entrySet()) {
            if (entry.getValue() != null && entry.getValue().size() > 0) {
                client.vgApi().updateVgMembers(entry.getKey(), StringUtil.join(entry.getValue(), ","));
                logger.info("updated vg {} members to {}", entry.getKey(), entry.getValue());
            }
        }
        CloudHostResult prevResult = client.hostApi().getByCloud(cloudId, 1, 10000);
        prevResult.getHosts().stream().forEach(h -> hostVoMap.put(h.getInnerIp(), h));
        List<ImportHostVo> host2Import = new ArrayList<>();
        for (HostVgVo hostVgVo : hosts) {
            CloudHostVo hostVo = hostVoMap.get(hostVgVo.getIp());
            if (hostVo != null) {
                logger.error("IP为{}（{}）的主机已经导入到当前云账户，忽略导入", hostVgVo.getIp(), hostVgVo.getInstanceName());
                continue;
            }
            ImportHostVo ihv = new ImportHostVo();
            ihv.setInnerIp(hostVgVo.getIp());
            ihv.setName(hostVgVo.getInstanceName());
            ihv.setOperatingSystem(hostVgVo.getPlatform());
            VirtualGroupVo groupVo = virtualGroupVoMap.get(hostVgVo.getVgL3());
            ihv.setDefaultVgIds(Arrays.asList(groupVo.getId()));
            host2Import.add(ihv);
        }
        if (host2Import.size() > 0) {
            for (ImportHostVo hostVo : host2Import) {
                ImportHostRequest ihr = new ImportHostRequest();
                ihr.setCloudId(cloudId);
                ihr.setTeamId(teamId);
                ihr.getHosts().add(hostVo);
                logger.info("开始导入主机 {}（{}） ", hostVo.getInnerIp(), hostVo.getName());
                long t = System.nanoTime();
                ImportHostResult hr = client.hostApi().importHosts(ihr);
                logger.info("imported host {} = {} using time {} seconds", hostVo.getInnerIp(), hr.getHostIds(), TimeUnit.NANOSECONDS.toSeconds(System.nanoTime() - t));
            }
        } else {
            logger.warn("No hosts need to be imported");
        }
    }

    private VirtualGroupVo getOrCreateVg(long teamId, Long parentVgId, String name) throws OpenapiException, DemoException {
        VirtualGroupVo vgv = virtualGroupVoMap.get(name);
        if (vgv != null) {
            return vgv;
        }
        VirtualGroupResult result = client.vgApi().getByName(teamId, name);
        if (result.getVgs().size() > 0) {
            VirtualGroupVo groupVo = result.getVgs().get(0);
            if (parentVgId == null && groupVo.getParentVgId() != null) {
                throw new DemoException("分组:" + name + "已经是子分组，不能作为顶级分组");
            } else if (parentVgId != null && !parentVgId.equals(groupVo.getParentVgId())) {
                throw new DemoException("分组:" + name + "存在不同的父级分组");
            }
            virtualGroupVoMap.put(name, groupVo);
            return groupVo;
        } else {
            VirtualGroupVo groupVo = client.vgApi().createVgNew(teamId, parentVgId, name, null, null);
            virtualGroupVoMap.put(name, groupVo);
            return groupVo;
        }
    }


}
