package com.cloudbility.openapi.demo.batch2;

import com.cloudbility.openapi.DemoException;
import com.cloudbility.openapi.OpenapiClient;
import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.common.StringUtil;
import com.cloudbility.openapi.demo.batch1.HostVgVo;
import com.cloudbility.openapi.request.ImportHostRequest;
import com.cloudbility.openapi.request.ImportUserRequest;
import com.cloudbility.openapi.response.*;
import com.cloudbility.openapi.vo.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

public class BatchImporter2 {
    private Logger logger = LoggerFactory.getLogger(BatchImporter2.class);
    private OpenapiClient client;
    //vgName->vgVo
    private Map<String, VirtualGroupVo> virtualGroupVoMap = new HashMap<>();
    //ip->hostVo
    private Map<String, CloudHostVo> hostVoMap = new HashMap<>();

    public BatchImporter2(OpenapiClient client) {
        this.client = client;
    }

    /**
     * 批量将主机、分组、用户导入到特定团队和云账号中
     *
     * @param teamId
     * @param cloudId
     * @param hosts
     * @throws Exception
     */
    public void importHostAndUser(long teamId, long cloudId, List<HostVgVo2> hosts) throws Exception {
        //1，创建分组
        for (HostVgVo2 hostVgVo : hosts) {
            VirtualGroupVo groupVo1 = getOrCreateVg(teamId, null, hostVgVo.getVgL1());
            VirtualGroupVo groupVo2 = getOrCreateVg(teamId, groupVo1.getId(), hostVgVo.getVgL2());
            VirtualGroupVo groupVo3 = getOrCreateVg(teamId, groupVo2.getId(), hostVgVo.getVgL3());
        }
        //vgId=>memberIds
        Map<Long, Set<String>> groupMemberMap = new HashMap<>();
        for (HostVgVo2 hostVgVo : hosts) {
            VirtualGroupVo groupVo = virtualGroupVoMap.get(hostVgVo.getVgL3());
            Set<String> list = groupMemberMap.get(groupVo.getId());
            if (list == null) {
                list = new HashSet<>();
            }
            list.addAll(hostVgVo.getAccounts());
            groupMemberMap.put(groupVo.getId(), list);
        }
        //2，校验账号是否存在
        Set<String> allAccounts = new HashSet<>();
        groupMemberMap.values().stream().forEach(s -> allAccounts.addAll(s));
        PagedUserListResult result = client.userApi().getUsersByTeam(teamId, 1, 10000);
        Set<String> existsAccounts = new HashSet<>(result.getUsers().stream().map(u -> u.getAccount()).collect(Collectors.toSet()));
        allAccounts.removeAll(existsAccounts);
        if (allAccounts.size() > 0) {
            throw new Exception("账号不存在：" + allAccounts);
        }
        //3,分组授予用户权限
        for (Map.Entry<Long, Set<String>> entry : groupMemberMap.entrySet()) {
            if (entry.getValue() != null && entry.getValue().size() > 0) {
                client.vgApi().updateVgMembers(entry.getKey(), StringUtil.join(entry.getValue(), ","));
                logger.info("updated vg {} members to {}", entry.getKey(), entry.getValue());
            }
        }
        //4,过滤未导入主机
        CloudHostResult prevResult = client.hostApi().getByCloud(cloudId, 1, 10000);
        prevResult.getHosts().stream().forEach(h -> hostVoMap.put(h.getInnerIp(), h));
        List<ImportHostVo> host2Import = new ArrayList<>();
        for (HostVgVo2 hostVgVo : hosts) {
            CloudHostVo hostVo = hostVoMap.get(hostVgVo.getIp());
            if (hostVo != null) {
                logger.error("IP为{}（{}）的主机已经导入到当前云账户，忽略导入", hostVgVo.getIp(), hostVgVo.getInstanceName());
                continue;
            }
            ImportHostVo ihv = new ImportHostVo();
            ihv.setInnerIp(hostVgVo.getIp());
            ihv.setName(hostVgVo.getInstanceName());
            ihv.setOperatingSystem(hostVgVo.getPlatform());
            VirtualGroupVo groupVo = virtualGroupVoMap.get(hostVgVo.getVgL3());
            ihv.setDefaultVgIds(Arrays.asList(groupVo.getId()));
            host2Import.add(ihv);
        }
        //5,导入未导入的主机
        if (host2Import.size() > 0) {
            for (ImportHostVo hostVo : host2Import) {
                ImportHostRequest ihr = new ImportHostRequest();
                ihr.setCloudId(cloudId);
                ihr.setTeamId(teamId);
                ihr.getHosts().add(hostVo);
                ihr.setVgIds(hostVo.getDefaultVgIds());
                logger.info("开始导入主机 {}（{}） ", hostVo.getInnerIp(), hostVo.getName());
                long t = System.nanoTime();
                ImportHostResult hr = client.hostApi().importHosts(ihr);
                logger.info("imported host {} = {} using time {} seconds", hostVo.getInnerIp(), hr.getHostIds(), TimeUnit.NANOSECONDS.toSeconds(System.nanoTime() - t));
            }
        } else {
            logger.warn("No hosts need to be imported");
        }
    }

    private VirtualGroupVo getOrCreateVg(long teamId, Long parentVgId, String name) throws OpenapiException, DemoException {
        VirtualGroupVo vgv = virtualGroupVoMap.get(name);
        if (vgv != null) {
            return vgv;
        }
        VirtualGroupResult result = client.vgApi().getByName(teamId, name);
        if (result.getVgs().size() > 0) {
            VirtualGroupVo groupVo = result.getVgs().get(0);
            if (parentVgId == null && groupVo.getParentVgId() != null) {
                throw new DemoException("分组:" + name + " 已经是子分组，不能作为顶级分组");
            } else if (parentVgId != null && !parentVgId.equals(groupVo.getParentVgId())) {
                throw new DemoException("分组:" + name + " 存在不同的父级分组");
            }
            virtualGroupVoMap.put(name, groupVo);
            return groupVo;
        } else {
            VirtualGroupVo groupVo = client.vgApi().createVgNew(teamId, parentVgId, name, null, null);
            virtualGroupVoMap.put(name, groupVo);
            return groupVo;
        }
    }


}
