package com.cloudbility.openapi.impl;

import com.cloudbility.openapi.AsyncScriptApi;
import com.cloudbility.openapi.OpenapiClient;
import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.ScriptListener;
import com.cloudbility.openapi.vo.JobVo;
import com.cloudbility.openapi.vo.ScriptExecutingHostsInfo;
import com.cloudbility.openapi.vo.ScriptRunResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.concurrent.*;

public class ScriptApiWrapper implements AsyncScriptApi {
    private static final Logger logger = LoggerFactory.getLogger(ScriptApiWrapper.class);
    private OpenapiClient openapiClient;
    private ScheduledExecutorService executorService = null;
    private long pollingInterval = 2000;//毫秒

    public ScriptApiWrapper(OpenapiClient openapiClient, ScheduledExecutorService executorService) {
        this.openapiClient = openapiClient;
        this.executorService = executorService;
    }

    public ScriptApiWrapper(OpenapiClient openapiClient, long pollingInterval, ScheduledExecutorService executorService) {
        this.openapiClient = openapiClient;
        this.pollingInterval = pollingInterval;
        if (pollingInterval < 1000) {
            throw new IllegalArgumentException("polling too frequently");
        }
        this.executorService = executorService;
    }

    @Override
    public JobVo executeScript(int scriptId, ScriptExecutingHostsInfo executingHostsInfo, ScriptListener listener) throws OpenapiException {
        JobVo jobVo = openapiClient.scriptApi().executeScript(scriptId, executingHostsInfo);
        PoolingJob poolingJob = new PoolingJob(openapiClient, jobVo, listener);
        Future future = executorService.scheduleWithFixedDelay(poolingJob, pollingInterval, pollingInterval, TimeUnit.MILLISECONDS);
        poolingJob.setFuture(future);
        return jobVo;
    }

    @Override
    public Future<ScriptRunResult> executeScriptAsync(int scriptId, ScriptExecutingHostsInfo executingHostsInfo) throws OpenapiException {
        JobVo jobVo = openapiClient.scriptApi().executeScript(scriptId, executingHostsInfo);
        ScriptRunResultFuture poolingJob = new ScriptRunResultFuture(openapiClient, jobVo);
        Future future = executorService.scheduleWithFixedDelay(poolingJob, pollingInterval, pollingInterval, TimeUnit.MILLISECONDS);
        poolingJob.setFuture(future);
        return poolingJob;
    }

    private class PoolingJob extends AbstractPoolingJob {

        private ScriptListener listener;

        public PoolingJob(OpenapiClient openapiClient, JobVo jobVo, ScriptListener listener) {
            super(openapiClient, jobVo);
            this.listener = listener;
        }

        @Override
        public void onDone(JobVo jobVo, ScriptRunResult result) {
            listener.onDone(jobVo, result);
        }

        @Override
        public void onError(JobVo jobVo, Throwable throwable) {
            listener.onError(jobVo, throwable);
        }
    }

    private class ScriptRunResultFuture extends AbstractPoolingJob implements Future<ScriptRunResult> {
        private BlockingQueue<Object> queue = new LinkedBlockingDeque<>();
        private volatile boolean done = false;

        public ScriptRunResultFuture(OpenapiClient openapiClient, JobVo jobVo) {
            super(openapiClient, jobVo);
        }

        @Override
        public void onDone(JobVo jobVo, ScriptRunResult result) {
            queue.offer(result);
            done = true;
        }

        @Override
        public void onError(JobVo jobVo, Throwable throwable) {
            queue.offer(throwable);
            done = true;
        }

        @Override
        public boolean cancel(boolean mayInterruptIfRunning) {
            return super.cancel();
        }

        @Override
        public boolean isCancelled() {
            return super.isCanceled();
        }

        @Override
        public boolean isDone() {
            return done;
        }

        @Override
        public ScriptRunResult get() throws InterruptedException, ExecutionException {
            Object obj = queue.take();
            if (obj instanceof ScriptRunResult) {
                return (ScriptRunResult) obj;
            }
            if (obj instanceof Throwable) {
                throw new ExecutionException((Throwable) obj);
            }
            return null;
        }

        @Override
        public ScriptRunResult get(long timeout, TimeUnit unit) throws InterruptedException, ExecutionException, TimeoutException {
            Object obj = queue.poll(timeout, unit);
            if (obj instanceof ScriptRunResult) {
                return (ScriptRunResult) obj;
            }
            if (obj instanceof Throwable) {
                throw new ExecutionException((Throwable) obj);
            }
            return null;
        }
    }

    public void setExecutorService(ScheduledExecutorService executorService) {
        this.executorService = executorService;
    }
}
