package com.cloudbility.openapi.v2;

import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.bind.NoneSAAS;
import com.cloudbility.openapi.request.AddLdapConfigRequest;
import com.cloudbility.openapi.response.LdapConfigVoList;
import com.cloudbility.openapi.response.LdapSyncConfigVoList;
import com.cloudbility.openapi.vo.CountResult;
import com.cloudbility.openapi.vo.LdapSyncConfigVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

@Api(tags = "16 system api", description = "系统管理API")
public interface SystemApi extends Openapi {

    @ApiOperation(value = "添加一条LDAP/AD配置")
    @RequestMapping(value = "/ldap/config", method = RequestMethod.POST)
    @NoneSAAS
    void addLdapConfig(@RequestBody AddLdapConfigRequest request) throws OpenapiException;

    @ApiOperation(value = "更新LDAP/AD配置（通过baseDn匹配）", notes = "注意，不能改变baseDn，即通过baseDn查找现有配置")
    @RequestMapping(value = "/ldap/config", method = RequestMethod.PUT)
    @NoneSAAS
    void updateLdapConfig(@RequestBody AddLdapConfigRequest request) throws OpenapiException;

    @ApiOperation(value = "更新LDAP/AD配置的BaseDn", notes = "注意，已经导入的用户全部在新的baseDn下进行认证，更换baseDn可能导致他们无法登陆。返回受影响的用户数。")
    @RequestMapping(value = "/ldap/config/baseDn", method = RequestMethod.PUT)
    @NoneSAAS
    CountResult updateLdapConfigBaseDn(@ApiParam(value = "旧的baseDn", required = true) @RequestParam("prevBaseDn") String prevBaseDn, @ApiParam(value = "新的baseDn", required = true) @RequestParam("newBaseDn") String newBaseDn) throws OpenapiException;

    @ApiOperation(value = "设置（或更新）同步配置")
    @RequestMapping(value = "/ldap/syncConfig", method = RequestMethod.PUT)
    @NoneSAAS
    void updateLdapSyncConfig(@RequestBody LdapSyncConfigVo request) throws OpenapiException;

    @ApiOperation(value = "获取系统LDAP/AD配置列表")
    @RequestMapping(value = "/ldap/configs", method = RequestMethod.GET)
    @NoneSAAS
    LdapConfigVoList getLdapConfig();

    @ApiOperation(value = "获取系统LDAP/AD同步配置列表")
    @RequestMapping(value = "/ldap/syncConfigs", method = RequestMethod.GET)
    @NoneSAAS
    LdapSyncConfigVoList getLdapSyncConfig();

    @ApiOperation(value = "删除LDAP/AD配置", notes = "通过这个配置导入的用户，账号会被回收，无法通过LDAP账号登录。管理控制台管理员可以通过手工指定登录账号和密码恢复使用。会删除对应的同步设置规则。")
    @RequestMapping(value = "/ldap/config", method = RequestMethod.DELETE)
    @NoneSAAS
    CountResult deleteLdapConfigBaseDn(@ApiParam(value = "baseDn", required = true) @RequestParam("baseDn") String prevBaseDn) throws OpenapiException;

    @ApiOperation(value = "绑定LDAP用户", notes = "设置ldapAccount时，会将LDAP中的特定账户绑定到指定行云管家用户；未设置ldapAccount时，将根据行云管家用户的账号查询LDAP，并进行绑定。ldapAccountKey可以指定LDAP中的属性名称，未设置时，从LDAP配置中读取")
    @RequestMapping(value = "/ldap/bind", method = RequestMethod.PUT)
    @NoneSAAS
    void bindLdapUser(@ApiParam(value = "baseDn", required = true) @RequestParam("baseDn") String baseDn,
                      @ApiParam(value = "用户ID", required = true) @RequestParam("userId") Long userId,
                      @ApiParam(value = "Ldap账号属性名。可选", required = false) @RequestParam(value = "ldapAccountKey", required = false) String ldapAccountKey,
                      @ApiParam(value = "Ldap账号。可选", required = false) @RequestParam(value = "ldapAccount", required = false) String ldapAccount
    ) throws OpenapiException;

    @ApiOperation(value = "解绑LDAP用户", notes = "newAccount未设置时，将生成随机账号，可以设置原来的账号，相当于保留账号，如果设置其他账号，不能与现有用户重复；<br/>" +
            "newPassword设置时，将按照系统的复杂度规则进行校验,未设置时，不会修改原有用户的本地密码；<br/>" +
            "generatePassword=true和newPassword不要同时设置；<br/>" +
            "forceUpdatePwd,emailNewPassword的默认取值：false，当newPassword设置时或者generatePassword=true时，emailNewPassword=true才会生效")
    @RequestMapping(value = "/ldap/unbind", method = RequestMethod.PUT)
    @NoneSAAS
    void unbindLdapUser(@ApiParam(value = "用户ID", required = true) @RequestParam("userId") Long userId,
                        @ApiParam(value = "解绑后的新账号。可选", required = false) @RequestParam(value = "newAccount", required = false) String newAccount,
                        @ApiParam(value = "解绑后的新密码。可选", required = false) @RequestParam(value = "newPassword", required = false) String newPassword,
                        @ApiParam(value = "解绑后生成随机密码。默认false", required = false) @RequestParam(value = "generatePassword", required = false, defaultValue = "false") boolean generatePassword,
                        @ApiParam(value = "下一次登录强制改密码", required = false) @RequestParam(value = "forceUpdatePwd", required = false, defaultValue = "false") boolean forceUpdatePwd,
                        @ApiParam(value = "是否邮件通知用户（如果设置了邮箱）", required = false) @RequestParam(value = "emailNewPassword", required = false, defaultValue = "false") boolean emailNewPassword
    ) throws OpenapiException;
}
