package com.cloudbility.openapi.v2;

import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.bind.NoneSAAS;
import com.cloudbility.openapi.bind.Permissions;
import com.cloudbility.openapi.request.ImportTeamRequest;
import com.cloudbility.openapi.response.*;
import com.cloudbility.openapi.vo.CountResult;
import com.cloudbility.openapi.vo.Permission;
import io.swagger.annotations.*;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

@Api(tags = "2 team api", description = "团队操作API")
public interface TeamApi extends Openapi {
    @ApiOperation(value = "获取所有团队ID", response = TeamBriefResult.class)
    @RequestMapping(value = "/team/findIds", method = RequestMethod.GET)
    @Permissions(Permission.TeamAccess)
    TeamBriefResult findTeamIds() throws OpenapiException;

    @ApiOperation(value = "获取团队信息", response = GetTeamResult.class)
    @RequestMapping(value = "/team/{id}", method = RequestMethod.GET)
    @Permissions(Permission.TeamAccess)
    GetTeamResult getTeam(@ApiParam(value = "团队ID", required = true) @PathVariable("id") Long id,
                                          @ApiParam(value = "是否获取成员列表") @RequestParam(value = "includeMembers", defaultValue = "true") boolean includeMembers
    ) throws OpenapiException;

    @ApiOperation(value = "获取团队信息（根据团队code）", response = GetTeamResult.class)
    @RequestMapping(value = "/team/byCode/{code}", method = RequestMethod.GET)
    @Permissions(Permission.TeamAccess)
    GetTeamResult getTeamByCode(@ApiParam(value = "团队code", required = true) @PathVariable("code") String code,
                                                @ApiParam(value = "是否获取成员列表") @RequestParam(value = "includeMembers", defaultValue = "true") boolean includeMembers
    ) throws OpenapiException;

    @ApiOperation(value = "获取团队列表（根据名称）", response = GetTeamResult.class)
    @RequestMapping(value = "/teams/byName", method = RequestMethod.GET)
    @Permissions(Permission.TeamAccess)
    TeamListResult getTeamByName(@ApiParam(value = "团队名称", required = true) @RequestParam("name") String name) throws OpenapiException;

    @ApiOperation(value = "获取团队列表（根据名称模糊查询）", response = GetTeamResult.class)
    @RequestMapping(value = "/teams/byNameLike", method = RequestMethod.GET)
    @Permissions(Permission.TeamAccess)
    TeamListResult getTeamByNameLike(@ApiParam(value = "团队名称", required = true) @RequestParam("name") String name) throws OpenapiException;

    @ApiOperation(value = "获取所有团队信息", response = GetTeamListResult.class)
    @RequestMapping(value = "/teams", method = RequestMethod.GET)
    @Permissions(Permission.TeamAccess)
    GetTeamListResult getTeams(@ApiParam(value = "页码") @RequestParam(value = "page", defaultValue = "1") int page,
                                               @ApiParam(value = "条数") @RequestParam(value = "size", defaultValue = "20") int size) throws OpenapiException;

    @ApiOperation(value = "导入团队", response = ImportTeamResult.class, notes = "注意：code字段不能设置为保留字，目前系统的保留字有：wx,mobile,assets,cloud_bootstrap,common,welcome,errorPage,flash,oldjs,pub,thirdparty_libs,downloads,help,cloudAdminer,cloudApiExecutor,cloudCi,cloudPortal,cloudTransit,portal_socketio,protected,rtunnel_srv_ws,imagepreview,api,proxy。")
    @RequestMapping(value = "/team", method = RequestMethod.POST)
    @Permissions(Permission.TeamAccess)
    @ApiResponses(value = {
            @ApiResponse(code = 201, message = "Team created successfully"),
            @ApiResponse(code = 409, message = "name or code conflict"),
    })
    ImportTeamResult importTeams(
            @ApiParam(value = "团队列表", required = true) @RequestBody ImportTeamRequest request) throws OpenapiException;


    @ApiOperation(value = "修改团队名称和所有者", response = OpenapiResponse.class, notes = "name和creator参数不能同时为null")
    @RequestMapping(value = "/team/{id}", method = RequestMethod.PUT)
    @Permissions(Permission.TeamAccess)
    void updateTeamNameOrCreator(
            @ApiParam(value = "团队ID", required = true) @PathVariable("id") Long id,
            @ApiParam(value = "团队名称，可选。如果设置，不能与当前ISV的其他团队重名。私有部署环境必须全局唯一。") @RequestParam(value = "name", required = false) String name,
            @ApiParam(value = "团队创建者用户ID,可选。如果设置，必须是当前ISV有权限的用户，私有部署环境可以是任何用户。修改团队创建者会进行转移身份操作。") @RequestParam(value = "creator", required = false) Long creator) throws OpenapiException;

    @ApiOperation(value = "修改团队名称和所有者（根据团队code）", response = OpenapiResponse.class, notes = "name和creator参数不能同时为null")
    @RequestMapping(value = "/team/byCode/{code}", method = RequestMethod.PUT)
    @Permissions(Permission.TeamAccess)
    void updateTeamNameOrCreatorByCode(
            @ApiParam(value = "团队code", required = true) @PathVariable("code") String code,
            @ApiParam(value = "团队名称，可选。如果设置，不能与当前ISV的其他团队重名。私有部署环境必须全局唯一。") @RequestParam(value = "name", required = false) String name,
            @ApiParam(value = "团队创建者账号,可选。如果设置，必须是当前ISV有权限的用户，私有部署环境可以是任何用户。修改团队创建者会进行转移身份操作。") @RequestParam(value = "creatorAccount", required = false) String creatorAccount) throws OpenapiException;

    @ApiOperation(value = "修改团队code", response = OpenapiResponse.class, notes = "name和creator参数不能同时为null")
    @RequestMapping(value = "/team/{id}/code", method = RequestMethod.PUT)
    @Permissions(Permission.TeamAccess)
    void updateTeamCode(
            @ApiParam(value = "团队ID", required = true) @PathVariable("id") Long id,
            @ApiParam(value = "团队code，必须是字符数字下划线的组合，即[A-Za-z0-9_]，全局唯一，不能与保留字段相同", required = true) @RequestParam(value = "code") String code) throws OpenapiException;

    @ApiOperation(value = "删除团队", response = OpenapiResponse.class)
    @RequestMapping(value = "/team/{ids}", method = RequestMethod.DELETE)
    @Permissions(Permission.TeamAccess)
    void deleteTeams(@ApiParam(value = "团队ID，用','连接", required = true) @PathVariable("ids") String idsStr) throws OpenapiException;

    @ApiOperation(value = "删除一个团队（根据团队code）", response = OpenapiResponse.class)
    @RequestMapping(value = "/team/byCode/{code}", method = RequestMethod.DELETE)
    @Permissions(Permission.TeamAccess)
    void deleteTeamsByCode(@ApiParam(value = "团队code", required = true) @PathVariable("code") String code) throws OpenapiException;

    @ApiOperation(value = "添加团队成员", response = OpenapiResponse.class)
    @RequestMapping(value = "/team/{id}/members/{userIds}", method = RequestMethod.POST)
    @Permissions(Permission.TeamAccess)
    void addMembers(
            @ApiParam(value = "团队ID", required = true) @PathVariable("id") Long id,
            @ApiParam(value = "用户id或者账号列表，用','连接", required = true) @PathVariable("userIds") String userIdsStr) throws OpenapiException;

    @ApiOperation(value = "添加一个团队成员（根据团队code和用户账号）", response = OpenapiResponse.class)
    @RequestMapping(value = "/team/byCode/{code}/members/{account:.+}", method = RequestMethod.POST)
    @Permissions(Permission.TeamAccess)
    void addMembersByCode(
            @ApiParam(value = "团队code", required = true) @PathVariable("code") String code,
            @ApiParam(value = "用户账号", required = true) @PathVariable("account") String account) throws OpenapiException;

    @ApiOperation(value = "删除团队成员", response = OpenapiResponse.class)
    @RequestMapping(value = "/team/{id}/members/{userIds}", method = RequestMethod.DELETE)
    @Permissions(Permission.TeamAccess)
    void removeMembers(
            @ApiParam(value = "团队ID", required = true) @PathVariable("id") Long id,
            @ApiParam(value = "用户id列表，用','连接", required = true) @PathVariable("userIds") String userIdsStr) throws OpenapiException;

    @ApiOperation(value = "删除一个团队成员（根据团队code和用户账号）", response = OpenapiResponse.class)
    @RequestMapping(value = "/team/byCode/{code}/members/{account:.+}", method = RequestMethod.DELETE)
    @Permissions(Permission.TeamAccess)
    void removeMembersByCode(
            @ApiParam(value = "团队code", required = true) @PathVariable("code") String idOrCode,
            @ApiParam(value = "用户账号", required = true) @PathVariable("account") String account) throws OpenapiException;

    @ApiOperation(value = "获取团队配额信息", response = GetTeamQuotaResult.class)
    @RequestMapping(value = "/team/{id}/quota", method = RequestMethod.GET)
    @Permissions(Permission.TeamAccess)
    @Deprecated
    GetTeamQuotaResult getTeamQuota(@ApiParam(value = "团队ID") @PathVariable("id") Long id) throws OpenapiException;

    @ApiOperation(value = "更新团队配额信息", response = GetTeamQuotaResult.class, notes = "仅在私有部署环境使用。返回更新后的配额信息。注意：成员配额不能超过License总配额，因为用户可以加入多个团队，所以团队的成员数仅受License用户数限制。而资源数配额必须满足所有团队资源总配额不能超过License资源配额。")
    @RequestMapping(value = "/team/{id}/quota", method = RequestMethod.PUT)
    @Permissions(Permission.TeamAccess)
    @Deprecated
    GetTeamQuotaResult updateTeamQuotaUsed(@ApiParam(value = "团队ID", required = true) @PathVariable("id") Long id,
                                                           @ApiParam(value = "团队成员数配额，不能超过License总配额，不设置时不更新") @RequestParam(value = "members", required = false) Integer members,
                                                           @ApiParam(value = "主机资源配额，不设置时不更新") @RequestParam(value = "hosts", required = false) Integer hosts,
                                                           @ApiParam(value = "CDN资源配额，不设置时不更新") @RequestParam(value = "cdns", required = false) Integer cdns,
                                                           @ApiParam(value = "Bucket资源配额（包括OSS、COS、S3等），不设置时不更新") @RequestParam(value = "buckets", required = false) Integer buckets,
                                                           @ApiParam(value = "数据库资源配额，不设置时不更新") @RequestParam(value = "dbs", required = false) Integer dbs) throws OpenapiException;

    @ApiOperation(value = "更新团队配额信息（根据团队code）", response = GetTeamQuotaResult.class, notes = "仅在私有部署环境使用。返回更新后的配额信息。注意：成员配额不能超过License总配额，因为用户可以加入多个团队，所以团队的成员数仅受License用户数限制。而资源数配额必须满足所有团队资源总配额不能超过License资源配额。")
    @RequestMapping(value = "/team/byCode/{code}/quota", method = RequestMethod.PUT)
    @Permissions(Permission.TeamAccess)
    @Deprecated
    @NoneSAAS
    GetTeamQuotaResult updateTeamQuotaByCode(@ApiParam(value = "团队code", required = true) @PathVariable("code") String code,
                                                             @ApiParam(value = "团队成员数配额，不能超过License总配额，不设置时不更新") @RequestParam(value = "members", required = false) Integer members,
                                                             @ApiParam(value = "主机资源配额，不设置时不更新") @RequestParam(value = "hosts", required = false) Integer hosts,
                                                             @ApiParam(value = "CDN资源配额，不设置时不更新") @RequestParam(value = "cdns", required = false) Integer cdns,
                                                             @ApiParam(value = "Bucket资源配额（包括OSS、COS、S3等），不设置时不更新") @RequestParam(value = "buckets", required = false) Integer buckets,
                                                             @ApiParam(value = "数据库资源配额，不设置时不更新") @RequestParam(value = "dbs", required = false) Integer dbs) throws OpenapiException;

    @ApiOperation(value = "获取团队配额信息（根据团队code）", response = GetTeamQuotaResult.class)
    @RequestMapping(value = "/team/byCode/{code}/quota", method = RequestMethod.GET)
    @Permissions(Permission.TeamAccess)
    @Deprecated
    GetTeamQuotaResult getTeamQuotaByCode(@ApiParam(value = "团队code", required = true) @PathVariable("code") String code) throws OpenapiException;

    @ApiOperation(value = "获取团队所有成员信息", response = PagedUserListResult.class)
    @RequestMapping(value = "/team/{id}/members", method = RequestMethod.GET)
    @Permissions(Permission.UserAccess)
    PagedUserListResult getMembers(@ApiParam(value = "团队ID", required = true) @PathVariable("id") Long id,
                                                   @ApiParam(value = "页码") @RequestParam(value = "page", defaultValue = "1") int page,
                                                   @ApiParam(value = "条数") @RequestParam(value = "size", defaultValue = "1000") int size) throws OpenapiException;

    @ApiOperation(value = "获取团队所有成员信息（根据团队code）", response = PagedUserListResult.class)
    @RequestMapping(value = "/team/byCode/{code}/members", method = RequestMethod.GET)
    @Permissions(Permission.UserAccess)
    PagedUserListResult getMembersByCode(@ApiParam(value = "团队code", required = true) @PathVariable("code") String code,
                                                         @ApiParam(value = "页码") @RequestParam(value = "page", defaultValue = "1") int page,
                                                         @ApiParam(value = "条数") @RequestParam(value = "size", defaultValue = "1000") int size) throws OpenapiException;

    @ApiOperation(value = "获取团队在线成员数")
    @RequestMapping(value = "/team/{id}/countOnlineMembers", method = RequestMethod.GET)
    @Permissions(Permission.UserAccess)
    CountResult countTeamMemberOnline(@ApiParam(value = "团队ID", required = true) @PathVariable("id") Long id) throws OpenapiException;

    @ApiOperation(value = "创建团队")
    @RequestMapping(value = "/team/createTeam", method = RequestMethod.POST)
    @Permissions(Permission.TeamAccess)
    ResponseEntity<GetTeamResult> createTeam(@ApiParam(value = "团队名称", required = true) @RequestParam(value = "teamName", required = true) String teamName,
                                                    @ApiParam(value = "团队标识", required = true) @RequestParam(value = "teamCode", required = true) String teamCode,
                                                    @ApiParam(value = "团队创建者", required = true) @RequestParam(value = "creatorId", required = true) Long creatorId) throws OpenapiException;
}
