package com.cloudbility.openapi.v2.client;

import com.cloudbility.openapi.OpenapiErrorCode;
import com.cloudbility.openapi.OpenapiException;
import com.cloudbility.openapi.client.OpenapiConfig;
import com.cloudbility.openapi.common.JsonUtil;
import com.cloudbility.openapi.common.SignUtil;
import com.cloudbility.openapi.common.StringUtil;
import com.cloudbility.openapi.v2.OpenapiOperation;
import feign.*;
import feign.codec.ErrorDecoder;
import feign.gson.GsonDecoder;

import java.io.InputStreamReader;
import java.text.SimpleDateFormat;
import java.util.*;

import static feign.FeignException.errorStatus;
import static feign.Retryer.NEVER_RETRY;

public class OpenapiClient implements OpenapiOperation {

    private OpenapiConfig config;
    private FeignClientConfig feignClientConfig;
    private boolean needSign = true;
    private Map<Class, Object> apis = new HashMap<>();


    public OpenapiClient(OpenapiConfig config, FeignClientConfig feignClientConfig) {
        this.config = config;
        this.feignClientConfig = feignClientConfig;
    }

    public boolean isNeedSign() {
        return needSign;
    }

    public void setNeedSign(boolean needSign) {
        this.needSign = needSign;
    }

    @Override
    public <T> T getApi(Class<T> clazz) {
        Object t = apis.get(clazz);
        if (t != null) {
            return (T) t;
        }
        synchronized (this) {
            t = apis.get(clazz);
            if (t != null) {
                return (T) t;
            }
            t = doCreateApi(clazz);
            apis.put(clazz, t);
            return (T) t;
        }
    }

    private <T> Object doCreateApi(Class<T> clazz) {
        Feign.Builder builder = Feign.builder();
        if (feignClientConfig.getClient() != null) {
            builder.client(feignClientConfig.getClient());
        }
        if (feignClientConfig.getContract() != null) {
            builder.contract(feignClientConfig.getContract());
        }
        if (feignClientConfig.getLogger() != null) {
            builder.logger(feignClientConfig.getLogger());
        }
        return builder
                .decoder(new GsonDecoder(JsonUtil.GSON))
                .decoder(new GsonDecoder(JsonUtil.GSON))
                .errorDecoder(new OpenapiErrorDecoder())
                .logLevel(Logger.Level.FULL)
                .retryer(NEVER_RETRY)
                .target(new DynamicAuthTokenTarget(clazz, config.getEndpoint()));
    }

    private class OpenapiErrorDecoder implements ErrorDecoder {
        @Override
        public Exception decode(String methodKey, Response response) {
            int statusCode = response.status();
            boolean ok = statusCode >= 200 && statusCode < 300;
            OpenapiErrorCode errorCode = null;
            Map map = null;
            if (!ok) {
                try {
                    map = JsonUtil.fromJsonAsMap(new InputStreamReader(response.body().asInputStream(), "UTF-8"));
                    if (map == null) {
                        map = Collections.emptyMap();
                    }
                    errorCode = OpenapiErrorCode.fromRequestResult(map);
                } catch (Throwable throwable) {
                    errorCode = OpenapiErrorCode.UnknownError("" + statusCode);
                }
                if (errorCode != null) {
                    String requestId = map != null ? (String) map.get("requestId") : null;
                    if (requestId != null) {
                        return new OpenapiException(statusCode, requestId, errorCode);
                    } else {
                        return new OpenapiException(statusCode, errorCode);
                    }
                }
            }
            return errorStatus(methodKey, response);
        }
    }

    private class DynamicAuthTokenTarget<T> extends Target.HardCodedTarget<T> {

        protected String timestampPattern = "yyyy-MM-dd'T'HH:mm:ss'Z'";

        public DynamicAuthTokenTarget(Class<T> type, String url) {
            super(type, type.getSimpleName(), url);
        }

        @Override
        public Request apply(RequestTemplate input) {
            if (!needSign) {//不需要签名的场景
                input.header("cloudbility-access-key-id", config.getAccessKeyId());
                return super.apply(input);
            }
            if (config.getNoAuthPaths() != null && config.getNoAuthPaths().contains(input.path())) {
                return input.request();
            }
            if (config.getTokenProvider() != null) {
                input.header("Authorization", config.getTokenProvider().getToken());
            } else {
                Map<String, Object> signMap = new HashMap<>();
                SimpleDateFormat sdf = new SimpleDateFormat(timestampPattern);
                signMap.put("timestamp", sdf.format(new Date()));
                signMap.put("version", config.getVersion());
                signMap.put("nonce", StringUtil.randomPasswordStr(6));
                signMap.put("accessKeyId", config.getAccessKeyId());
                Map<String, Collection<String>> mkv = input.queries();
                if (mkv != null) {
                    for (Map.Entry<String, Collection<String>> entry : mkv.entrySet()) {
                        if (signMap.containsKey(entry.getKey())) {
                            try {
                                throw new OpenapiException(OpenapiErrorCode.InvalidParameter("参数" + entry.getKey() + "重复"));
                            } catch (OpenapiException e) {
                                throw new RuntimeException(e);
                            }
                        }
                        Collection<String> value = entry.getValue();
                        //TODO
//                        if (request.isMultipart() && value instanceof MultipartPart) {
//                            continue;
//                        }
                        if (value.size() == 1) {
                            signMap.put(entry.getKey(), value.iterator().next());
                        } else {
                            signMap.put(entry.getKey(), new ArrayList(value));
                        }
                    }
                }
                if (StringUtil.isEmpty(config.getAccessKeySecret())) {
                    throw new IllegalArgumentException("cannot sign,accessKeySecret not set");
                }
                String signature = SignUtil.sign(input.path(), signMap, input.method().toUpperCase(), config.getAccessKeySecret());
                signMap.put("signature", signature);
                signMap.forEach((k, v) -> {
                    if (SignUtil.getSigningParameterNames().contains(k)) {
                        input.query(k, Arrays.asList(String.valueOf(v)));
                    }
                });
            }
            //父类拼接URL
            return super.apply(input);
        }
    }


}
